package org.processmining.visualisation3d;

import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Vector;

import org.processmining.visualisation3d.gldatastructures.JoglTexture;

public class FontData 
{
	private String texResLoc;
	
	private JoglTexture tex;
	private Vector<Float> cleft = new Vector<Float>();
	private Vector<Float> cright = new Vector<Float>();
	private Vector<Float> cdiff = new Vector<Float>();
	
	public FontData(String dataFileLoc, String texFileLoc)
	{
		texResLoc = "/org/processmining/processprofiler3d/compviz/jogl/resources/FreeSans.tga";
		ReadFntData("/org/processmining/processprofiler3d/compviz/jogl/resources/FreeSans.fdt");
		
		texResLoc = texFileLoc;
		ReadFntData(dataFileLoc);
	}
	
	public void LoadFontTexture(GraphicsPipeline pipe)
	{
		tex = JoglTexture.LoadTextureFromFile(pipe, texResLoc, true);
		JoglResourceManager.AddResource("FontTexture", tex);
	}
	
	public void DisposeFontTexture(GraphicsPipeline pipe)
	{
		tex.Dispose(pipe);
		tex = null;
	}
	
	private void ReadFntData(String resLocation)
	{
		cleft.clear();
		cright.clear();
		cdiff.clear();

		System.out.println("Loading font data: " + resLocation);
		
		//InputStream istream = ResourceAccessor.getInputStream(resLocation);
		InputStream istream = this.getClass().getResourceAsStream(resLocation);

        // create new data input stream
		DataInputStream dis = new DataInputStream(istream);

		try 
		{
			//DecimalFormat df = new DecimalFormat("#.######");
			
			for (int i = 0; i < 256; i++)
			{
				//read two float in little endian order
				float fl = ReadFloatLE(dis);
				float fr = ReadFloatLE(dis);

				//System.out.println("Font data: " + df.format(fl) + "/" + df.format(fr));
				
				cleft.add(fl);
				cright.add(fr);
				cdiff.add((fl + fr) * 16.0f);
			}
			dis.close();
			istream.close();
		} 
		catch (IOException e) 
		{
			System.out.println("Error loading font data!");
		}	
	}
	
	//read floating point number in little endian order
	private float ReadFloatLE(DataInputStream dis)
	{
		byte[] data1 = new byte[4];
		for (int j = 0; j < 4; j++)
		{
			try 
			{
				data1[j] = dis.readByte();
			} catch (IOException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		return ByteBuffer.wrap(data1).order(ByteOrder.LITTLE_ENDIAN).getFloat();
	}
	
	public JoglTexture getTexture()
	{
		return tex;
	}
	
	public JoglTexture getTexture(GraphicsPipeline pipe)
	{
		if (tex == null)
		{
			LoadFontTexture(pipe);
		}
		return tex;
	}
	
	public double GetCharLeft(char c)
	{
		int idx = ((int)c) - 32;
		return cleft.get(idx);
	}
	
	public double GetCharRight(char c)
	{
		int idx = ((int)c) - 32;
		return cright.get(idx);
	}
	
	public double GetCharWidth(char c, double width)
	{
		if (c == ' ')
		{
			return width * 0.25;
		}
		else
		{
			int idx = ((int)c) - 32;
//			double left = cleft.get(idx);
//			double right = cright.get(idx);
//			double diff = ((left + right) * 16.0) * width;
			double diff = cdiff.get(idx) * width;
			return width - diff;
		}	
	}

}
