package org.processmining.visualisation3d;

import org.processmining.visualisation3d.gldatastructures.JoglMultiSampleFrameBufferObject;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTarget;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTargetFramebuffer;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTargetGLCanvas;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.measurements.AverageTimeMeasure;
import org.processmining.visualisation3d.settings.JoglGraphicsSettings;
import org.processmining.visualisation3d.utils.IScreenshotCapture;
import org.processmining.visualisation3d.utils.IVRAdapter;
import org.processmining.visualisation3d.utils.JoglScreenshotCapture;
import org.processmining.visualisation3d.utils.JoglScreenshotCaptureHQ;

import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.GL2ES1;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.GLCapabilities;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.awt.GLCanvas;
import com.jogamp.opengl.fixedfunc.GLLightingFunc;

public class GraphicsDisplay 
{
	public JoglRenderTarget target = null;
	
	private int width = 800;
	private int height = 600;
	//private int samples = 8;
	private JoglGraphicsSettings graphicsSettings = null;
	
	private JoglMultiSampleFrameBufferObject fbo = null;
	
	//private AverageTimeMeasure avgUpdateTime = new AverageTimeMeasure();
	private AverageTimeMeasure avgRenderTime = new AverageTimeMeasure();
	
	//private boolean bScreenshot = false;
	private IScreenshotCapture screenshotCapture = null;

	public IVRAdapter myVR = null;
	
	private boolean bFramebufferInitFailed = false;

	public AverageTimeMeasure getAvgRenderTime()
	{
		return avgRenderTime;
	}
	
	public JoglRenderTarget getTarget()
	{
		return target;
	}
	
	public GraphicsDisplay()
	{
	}
	
	public void setGraphicsSettings(JoglGraphicsSettings jgs)
	{
		graphicsSettings = jgs;
	}
	
	public static GLCanvas CreateGLCanvas()
	{
		GLCanvas canvas = null;
		
		//getting the capabilities object of GL2 profile
		final GLProfile profile = GLProfile.get(GLProfile.GL2);
		//System.out.println(GLProfile.glAvailabilityToString());
		GLCapabilities capabilities = new GLCapabilities(profile);
		capabilities.setDepthBits(24);
		capabilities.setDoubleBuffered( true );
		capabilities.setHardwareAccelerated( true );
		int dbits = capabilities.getDepthBits();
		
		System.out.println("GL depth precision: " + dbits + "bits");
		
		try
		{
			canvas = new GLCanvas(capabilities);
		}
		catch (Exception ex)
		{
			System.out.println("Exception when trying to create GLCanvas: " + ex.getMessage());
		}
		
		return canvas;
	}
	
	public void Screenshot(String fileName)
	{
		//bScreenshot = true;
		screenshotCapture = new JoglScreenshotCapture(fileName);
	}
	 
	public void Screenshot(String fileName, int resx, int resy, int samples)
	{
		//bScreenshot = true;
		screenshotCapture = new JoglScreenshotCaptureHQ(fileName, resx, resy, samples);
	}
	
	public void Screenshot(IScreenshotCapture cap)
	{
		screenshotCapture = cap;
	}
	
	public void init(GLAutoDrawable glDrawable, GraphicsPipeline pipe) 
	{
		System.out.println("Initializing GLCanvas...");
		final GL2 gl = glDrawable.getGL().getGL2();
		
		gl.glShadeModel(GLLightingFunc.GL_SMOOTH);
		gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
		gl.glClearDepth(1.0f);
		gl.glEnable(GL.GL_DEPTH_TEST);
		gl.glDepthFunc(GL.GL_LEQUAL);
		gl.glHint(GL2ES1.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);
		gl.setSwapInterval(0);	//deactivate VSync!
		
		if (graphicsSettings.GetAntiAliasingSamples() > 1)
		{
			target = initMultiSampleFBO(pipe, width, height, graphicsSettings.GetAntiAliasingSamples());
		}

		if (target == null)
		{
			target = new JoglRenderTargetGLCanvas(glDrawable);
		}
	}
	
	private JoglRenderTargetFramebuffer initMultiSampleFBO(GraphicsPipeline pipe, int width, int height, int samples) 
	{
		//System.out.println("Trying to initialize framebuffer: " + width + "x" + height + "x" + samples);
		
		fbo = new JoglMultiSampleFrameBufferObject(pipe, width, height, samples);
		fbo.Bind(pipe);
		fbo.AttachDepthBuffer(pipe, fbo.CreateRboAttachment(pipe, GL.GL_DEPTH_COMPONENT32));
		fbo.AttachColorBuffer0(pipe, fbo.CreateRboAttachment(pipe, GL.GL_RGBA8));

		if (!fbo.isFramebufferComplete())
		{
			System.out.println("Framebuffer (" + width + "x" + height + "x" + samples + ") failed to initialize.");
			fbo.Unbind(pipe);
			fbo.Dispose(pipe);
			fbo = null;
			
			bFramebufferInitFailed = true;
			return null;
		}
		
		fbo.Unbind(pipe);
		return new JoglRenderTargetFramebuffer(fbo);
	}
	
	public void display(GLAutoDrawable glDrawable, GraphicsPipeline pipe) 
	{
        //System.out.println("display");
		try
		{
			avgRenderTime.start();
			
			if (myVR == null)
			{
				RenderScene(glDrawable, pipe);
			}
			else
			{
				RenderSceneVR(glDrawable, pipe);
			}
			
			avgRenderTime.stop();
		}
		catch (Exception ex)
		{
			System.out.println("Error in scene render! " + ex.getMessage());
			//System.out.println(ex.getMessage());
			//System.out.println(ex.toString());
		}
		//MeasureFrameTime();
	}
	
	private void RenderScene(GLAutoDrawable glDrawable, GraphicsPipeline pipe) 
	{
		if (fbo != null && fbo.getSampleNum() != graphicsSettings.GetAntiAliasingSamples())
		{
			fbo.Dispose(pipe);
			fbo = null;
			target = new JoglRenderTargetGLCanvas(glDrawable);
		}
		if (fbo == null && graphicsSettings.GetAntiAliasingSamples() > 1 && !bFramebufferInitFailed)
		{
			target = initMultiSampleFBO(pipe, width, height, graphicsSettings.GetAntiAliasingSamples());
			
			if (target == null)
			{
				target = new JoglRenderTargetGLCanvas(glDrawable);
			}
		}
		
		if (target != null)
		{
			pipe.target = target;
			
			//set scene background
			JoglVectord3 bgColor = pipe.scene.getBackgroundColor();
			target.setBackgroundColor(bgColor);
			
			target.Bind(pipe);
			
			target.Clear(pipe);
			
//			pipe.cam.logCameraMatrix();
//			pipe.cam.logProjectionMatrix();

			pipe.scene.Draw(pipe);
			
			target.Unbind(pipe);
			
			if (fbo != null)
			{
				fbo.BlitToBackbuffer(pipe, width, height);
			}
			
			if (screenshotCapture != null)
			{
				//bScreenshot = false;
				screenshotCapture.GrabScreenshot(pipe);
				//JoglCapture.GrabScreenshotHQ(gl, scene, 8192, 8192, 4);
				//JoglCapture.RenderCubeMap(gl, scene);
				screenshotCapture = null;
			}
			
			target.FitViewport(pipe);
		}
	}

	private void RenderSceneVR(GLAutoDrawable glDrawable, GraphicsPipeline pipe) 
	{
		//pipe.target = target;
		pipe.target = new JoglRenderTargetGLCanvas(glDrawable);
		//set scene background
		JoglVectord3 bgColor = pipe.scene.getBackgroundColor();
		target.setBackgroundColor(bgColor);
		
		myVR.Render(pipe);
	}
	
	public void reshape(GLAutoDrawable glDrawable, int x, int y, int width, int height, GraphicsPipeline pipe) 
	{
		final GL2 gl = glDrawable.getGL().getGL2();
		
		this.width = width;
		this.height = height;
		
		if (fbo != null && !bFramebufferInitFailed)
		{
			fbo.Dispose(pipe);
			target = initMultiSampleFBO(pipe, width, height, graphicsSettings.GetAntiAliasingSamples());
			
			if (target == null)
			{
				target = new JoglRenderTargetGLCanvas(glDrawable);
			}
		}
		else
		{
			target = new JoglRenderTargetGLCanvas(glDrawable);
		}
		
		//initialize viewport
		gl.glViewport(x, y, width, height);
	}
	
	public void dispose(GLAutoDrawable gLDrawable) 
	{
//		GL2 gl = gLDrawable.getGL().getGL2();
//
//		if (fbo != null)
//		{
//			fbo.Dispose(gl);
//		}
	}
}
