package org.processmining.visualisation3d;

import java.util.Stack;

import org.processmining.visualisation3d.garbagecollection.GLGarbageCollector;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTarget;
import org.processmining.visualisation3d.graphicsdatastructures.JoglScissorRect;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.settings.JoglGraphicsSettings;

import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.fixedfunc.GLMatrixFunc;

public class GraphicsPipeline 
{
	public GL2 gl;
	public GraphicsDisplay display;
	public JoglScene scene;
	public JoglCamera cam;
	public JoglRenderTarget target;
	
	public JoglTransform transform = new JoglTransform(this, GLMatrixFunc.GL_MODELVIEW);
	public JoglTransform projection = new JoglTransform(this, GLMatrixFunc.GL_PROJECTION);
	
	public Stack<PipelineStateGL2> stateStack = new Stack<PipelineStateGL2>();
	
	public GLGarbageCollector gc;
	public JoglGraphicsSettings settings;
	
	public GraphicsPipeline()
	{
	}
	
	public GraphicsPipeline(GraphicsPipeline p)
	{
		gl = p.gl;
		display = p.display;
		scene = p.scene;
		cam = p.cam;
		target = p.target;
		gc = p.gc;
		stateStack = p.stateStack;
		settings = p.settings;
	}
	
	public JoglGraphicsSettings getGraphicsSettings()
	{
		if (settings == null)
		{
			settings = JoglGraphicsSettings.GetDefaultSettings(gl);
		}
		return settings;
	}
	
	public void SetScissorRect(JoglScissorRect srect)
	{
		if (srect != null)
		{
			gl.glEnable(GL.GL_SCISSOR_TEST);
			gl.glScissor(srect.getX(), srect.getY(), srect.getWidth(), srect.getHeight());
		}
		else
		{
			gl.glDisable(GL.GL_SCISSOR_TEST);
		}
	}
	
	public void PushPipelineState(PipelineStateGL2 ps)
	{
		stateStack.push(ps);
		
		ps.Apply(this);
	}
	
	public void PopPipelineState(PipelineState ps)
	{
		if (stateStack.peek() != ps)
		{
			System.out.println("Trying to unapply pipeline state that is not currently on top of the state stack!");
		}
		
		if (!stateStack.isEmpty())
		{
			stateStack.peek().Unapply(this);
		}
		
		stateStack.pop();
	}
}
