package org.processmining.visualisation3d;

import java.util.Iterator;
import java.util.Vector;

public class JoglInteractionCollection implements Iterable<JoglInteraction>
{
	private Vector<JoglInteraction> interactions = new Vector<JoglInteraction>();

	public void add(JoglInteraction ia)
	{
		//sort objects in collection by zlevel value
		for (int i = 0; i < interactions.size(); i++)
		{
			if (ia.getZLevel() < interactions.get(i).getZLevel())
			{
				interactions.insertElementAt(ia, i);
				return;
			}
		}

		//if not inserted before existing element add as last element
		interactions.add(ia);
	}
	
	public JoglInteraction get(int i)
	{
		return interactions.get(i);
	}
	
	public void remove(JoglInteraction ia)
	{
		interactions.remove(ia);
	}
	
	public void clear()
	{
		interactions.clear();
	}
	
	public <T extends JoglInteraction> T find(Class<T> c)
	{
		for (JoglInteraction ji : interactions)
		{
			if (c.isInstance(ji))
			{
				return (T)ji;
			}
		}
		return null;
	}
	
	public int size()
	{
		return interactions.size();
	}

	public Iterator<JoglInteraction> iterator() 
	{
		return interactions.iterator();
	}
	
	public String printStack()
	{
		String s = "Current Interactions:\r\n";
		
		for (JoglInteraction ji : interactions)
		{
			s += "  " + ji.zlevel + ": " + ji.toString() + "\r\n";
		}
		
		return s;
	}
}
