package org.processmining.visualisation3d;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Vector;

import org.processmining.visualisation3d.input.JoglInputDisplaySizeChangedEvent;
import org.processmining.visualisation3d.input.JoglInputKeyEvent;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.input.JoglInputMouseWheelEvent;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.utils.IScreenshotCapture;

import com.jogamp.opengl.awt.GLCanvas;
import com.jogamp.opengl.util.Animator;
import com.jogamp.opengl.util.AnimatorBase;
import com.jogamp.opengl.util.FPSAnimator;

/**
 * GL interaction controller
 * Listens for user input and redraws the scene whenever necessary
 */
public class JoglInteractionManager implements KeyListener, IMouseProcessor 
{
	private JoglScene scene = null;
	//private FPSAnimator anim = null;
	private AnimatorBase anim = null;
	private GraphicsDisplay display = null;
	
	private boolean bAnimPaused = true;
	
	private JoglInteractionCollection interactions = new JoglInteractionCollection();
	
	public JoglInteractionManager(GraphicsDisplay disp)
	{
		display = disp;
	}
	
	public AnimatorBase getAnimator()
	{
		return anim;
	}
	
//	public GraphicsDisplay getDisplay()
//	{
//		return display;
//	}
	
	private Vector<JoglInteraction> getInteractions()
	{
		Vector<JoglInteraction> lst = new Vector<JoglInteraction>();
		for (JoglInteraction ji : interactions)
		{
			lst.add(ji);
		}
		return lst;
	}
	
	public void addInteraction(JoglInteraction i)
	{
	  	interactions.add(i);
	}
		
	public <T extends JoglInteraction> T findInteraction(Class<T> c)
	{
		return interactions.find(c);
	}
	  
	public void removeInteraction(JoglInteraction i)
	{
	  	interactions.remove(i);
	}
	
	public void setScene(JoglScene s)
	{
		scene = s;
	}
	 
	public void Screenshot(String fileName)
	{
		display.Screenshot(fileName);
	}
	 
	public void Screenshot(String fileName, int resx, int resy, int samples)
	{
		display.Screenshot(fileName, resx, resy, samples);
	}
	
	public void Screenshot(IScreenshotCapture cap)
	{
		display.Screenshot(cap);
	}
	
//	private int updateCounter = 0;

	//private AverageTimeMeasure avgSubmitTime = new AverageTimeMeasure();
	//private int count = 0;
	
	//public IVRAdapter myVR = null;
 
	
	public void UpdateScene(JoglScene scene)
	{
    	for (JoglInteraction ji : getInteractions())
    	{
    		ji.UpdateScene(scene);
    	}
    	
    	CheckIfStopAnim();
	}

	//KeyListener method implementations
	public void keyPressed(KeyEvent evt) 
	{
        //System.out.println(evt);
        JoglInputKeyEvent jike = new JoglInputKeyEvent(evt);
    	for (JoglInteraction ji : getInteractions())
    	{
    		if (ji.keyPressed(jike))
    		{
    			break;
    		}
    	}
    	
    	CheckIfResumeAnim();
	}

	public void keyReleased(KeyEvent evt) 
	{
        //System.out.println(evt); 

        JoglInputKeyEvent jike = new JoglInputKeyEvent(evt);
    	for (JoglInteraction ji : getInteractions())
    	{
    		if (ji.keyReleased(jike))
    		{
    			break;
    		}
    	}
    	
    	CheckIfResumeAnim();
	}

	public void keyTyped(KeyEvent evt) 
	{
        //System.out.println(evt); 
        JoglInputKeyEvent jike = new JoglInputKeyEvent(evt);
    	for (JoglInteraction ji : getInteractions())
    	{
    		if (ji.keyTyped(jike))
    		{
    			break;
    		}
    	}
    	
    	CheckIfResumeAnim();
	}
 
	//MouseListener method implementation
    /* (non-Javadoc)
	 * @see org.processmining.visualisation3d.IMouseProcessor#mouseClicked(java.awt.event.MouseEvent)
	 */
    @Override
	public void mouseClicked(JoglInputMouseEvent jime) 
    {
        //System.out.println(interactions.printStack()); 
    	for (JoglInteraction ji : getInteractions())
    	{
    		if (ji.mouseClicked(jime))
    		{
    			break;
    		}
    	}
    	
    	CheckIfResumeAnim();
    }

    /* (non-Javadoc)
	 * @see org.processmining.visualisation3d.IMouseProcessor#mouseEntered(java.awt.event.MouseEvent)
	 */
    @Override
	public void mouseEntered(JoglInputMouseEvent jime) 
    {
    	for (JoglInteraction ji : getInteractions())
    	{
    		ji.mouseEntered(jime);
    	}
    	
    	CheckIfResumeAnim();
    }

    /* (non-Javadoc)
	 * @see org.processmining.visualisation3d.IMouseProcessor#mouseExited(java.awt.event.MouseEvent)
	 */
    @Override
	public void mouseExited(JoglInputMouseEvent jime) 
    {
    	for (JoglInteraction ji : getInteractions())
    	{
    		ji.mouseExited(jime);
    	}
    	
    	CheckIfResumeAnim();
    }

    /* (non-Javadoc)
	 * @see org.processmining.visualisation3d.IMouseProcessor#mousePressed(java.awt.event.MouseEvent)
	 */
    @Override
	public void mousePressed(JoglInputMouseEvent jime) 
    {
      //System.out.println("mouse pressed: " + e.getButton());
    	for (JoglInteraction ji : getInteractions())
    	{
    		if (ji.mousePressed(jime))
    		{
    			break;
    		}
    	}
    	
    	CheckIfResumeAnim();
    }

    /* (non-Javadoc)
	 * @see org.processmining.visualisation3d.IMouseProcessor#mouseReleased(java.awt.event.MouseEvent)
	 */
    @Override
	public void mouseReleased(JoglInputMouseEvent jime) 
    {
        //System.out.println("mouse released: " + e.getButton());
    	for (JoglInteraction ji : getInteractions())
    	{
    		if (ji.mouseReleased(jime))
    		{
    			break;
    		}
    	}
    	
    	//System.out.print(interactions.printStack());
    	
    	CheckIfResumeAnim();

    }

    /* (non-Javadoc)
	 * @see org.processmining.visualisation3d.IMouseProcessor#mouseMoved(java.awt.event.MouseEvent)
	 */
    @Override
	public void mouseMoved(JoglInputMouseEvent jime) 
    {
        //System.out.println("mouse moved");
    	for (JoglInteraction ji : getInteractions())
    	{
    		ji.mouseMoved(jime);
    	}
    	
    	CheckIfResumeAnim();
    }

    /* (non-Javadoc)
	 * @see org.processmining.visualisation3d.IMouseProcessor#mouseDragged(java.awt.event.MouseEvent)
	 */
    @Override
	public void mouseDragged(JoglInputMouseEvent jime) 
    {
        //System.out.println("mouse dragged");
    	for (JoglInteraction ji : getInteractions())
    	{
    		ji.mouseDragged(jime);
    	}
    	
    	CheckIfResumeAnim();
    }

	/* (non-Javadoc)
	 * @see org.processmining.visualisation3d.IMouseProcessor#mouseWheelMoved(java.awt.event.MouseWheelEvent)
	 */
	@Override
	public void mouseWheelMoved(JoglInputMouseWheelEvent jimwe) 
	{
        //System.out.println("mouse wheel moved");
    	for (JoglInteraction ji : getInteractions())
    	{
    		ji.mouseWheelMoved(jimwe);
    	}
    	
    	CheckIfResumeAnim();
	}

	public void displaySizeChangedEvent(int w, int h)
	{
		JoglInputDisplaySizeChangedEvent jidsce = new JoglInputDisplaySizeChangedEvent(w, h);
    	for (JoglInteraction ji : getInteractions())
    	{
    		ji.displaySizeChanged(jidsce);
    	}
    	
    	CheckIfResumeAnim();
	}
	
	private void CheckIfResumeAnim() 
	{
		if (bAnimPaused)
    	{
			//never pause in VR mode
//			if (display != null && display.myVR != null)
//			{
//				anim.resume();
//				bAnimPaused = false;
//			}
//			else
//			{
	        	if (ShouldResumeAnim())
	        	{
					anim.resume();
					bAnimPaused = false;
	        	}
//			}
    	}
	}

	public boolean ShouldResumeAnim() 
	{
		for (JoglInteraction ji : getInteractions())
		{
			if (ji.ShouldAnimate())
			{
				//System.out.println("Resume Animation.");
				return true;
			}
		}
		return false;
	}

	private void CheckIfStopAnim() 
	{
		return;
//		if (!bAnimPaused)
//    	{
//			//don't stop animating in VR mode
//			if (myVR != null)
//			{
//				return;
//			}
//			
//			boolean bShouldPause = true;
//        	for (JoglInteraction ji : getInteractions())
//	    	{
//        		if (ji.ShouldAnimate())
//        		{
//        			bShouldPause = false;
//        			break;
//        		}
//        	}
//        	if (bShouldPause)
//        	{
//    			//System.out.println("Pause Animation.");
//        		anim.pause();
//        		bAnimPaused = true;
//        	}
//    	}
	}
	
	public void InitAnimator(GLCanvas canvas)
	{
		if (anim != null)
		{
			anim.stop();
			anim.remove(canvas);
		}
		
		//anim = new FPSAnimator(canvas, 30);
		anim = new FPSAnimator(canvas, 90);
		//anim = new Animator(canvas);
		//((Animator)anim).setRunAsFastAsPossible(true);
		//anim.start();
		//anim.pause();
	}
	
	public void InitVRAnimator(GLCanvas canvas)
	{	
		if (anim != null)
		{
			anim.stop();
			anim.remove(canvas);
		}
		
		anim = new Animator(canvas);
		((Animator)anim).setRunAsFastAsPossible(true);

		anim.start();
		bAnimPaused = false;
	}
	
	public void ClearInteractions()
	{
		interactions.clear();
	}
}
