package org.processmining.visualisation3d;

import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord3;

public class JoglTransform
{
	private static int matrixMode = -1;
	
	private GraphicsPipeline pipe;
	private int myMatrixMode = -1;
	
	public JoglTransform(GraphicsPipeline gp, int mMode)
	{
		pipe = gp;
		myMatrixMode = mMode;
	}
	
	public void Push()
	{
		CheckMatrixMode();
		
		pipe.gl.glPushMatrix();
	}
	
	public void Pop()
	{
		CheckMatrixMode();
		
		pipe.gl.glPopMatrix();
	}
	
	public void MultMatrix(JoglMatrixd4x4 mat)
	{
		CheckMatrixMode();
		
		pipe.gl.glMultMatrixd(mat.GetData(), 0);
	}
	
	public void LoadMatrix(JoglMatrixd4x4 mat)
	{
		CheckMatrixMode();
		
		pipe.gl.glLoadMatrixd(mat.GetData(), 0);
	}
	
	public void LoadIdentity()
	{
		CheckMatrixMode();
		
		pipe.gl.glLoadIdentity();
	}
	
	public void Translated(JoglVectord3 v)
	{
		CheckMatrixMode();
		
		pipe.gl.glTranslated(v.getX(), v.getY(), v.getZ());
	}
	
	public void Translated(double x, double y, double z)
	{
		CheckMatrixMode();
		
		pipe.gl.glTranslated(x, y, z);
	}
	
	public void Translatef(JoglVectord3 v)
	{
		CheckMatrixMode();
		
		pipe.gl.glTranslatef((float)v.getX(), (float)v.getY(), (float)v.getZ());
	}
	
	public void Scaled(JoglVectord3 v)
	{
		CheckMatrixMode();
		
		pipe.gl.glScaled(v.getX(), v.getY(), v.getZ());
	}
	
	public void Scaled(double x, double y, double z)
	{
		CheckMatrixMode();
		
		pipe.gl.glScaled(x, y, z);
	}
	
	public void Scalef(JoglVectord3 v)
	{
		CheckMatrixMode();
		
		pipe.gl.glScalef((float)v.getX(), (float)v.getY(), (float)v.getZ());
	}

	private void CheckMatrixMode()
	{
		if (matrixMode != myMatrixMode)
		{
			pipe.gl.glMatrixMode(myMatrixMode);
			matrixMode = myMatrixMode;
		}
	}

}
