package org.processmining.visualisation3d;

import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.maths.JoglVectord4;

import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;

public class PipelineStateGL2 extends PipelineState
{
	protected GL2 gl = null;
	
	public PipelineStateGL2()
	{
	}

	@Override
	public void Apply(GraphicsPipeline pipe)
	{
		super.Apply(pipe);
		
		gl = pipe.gl;
	}

	@Override
	public void Unapply(GraphicsPipeline pipe)
	{
		super.Unapply(pipe);
		
		gl = pipe.gl;
	}

	
	protected void setColor(float r, float g, float b)
	{
		gl.glColor3f(r, g, b);
	}
	
	protected void setColor(double r, double g, double b)
	{
		gl.glColor3d(r, g, b);
	}
	
	protected void setColor(JoglVectord3 col)
	{
		gl.glColor3dv(col.getData(), 0);
	}
	
	protected void setColor(float r, float g, float b, float a)
	{
		gl.glColor4f(r, g, b, a);
	}
	
	protected void setColor(double r, double g, double b, double a)
	{
		gl.glColor4d(r, g, b, a);
	}
	
	protected void setColor(JoglVectord4 col)
	{
		gl.glColor4dv(col.getData(), 0);
	}
	
//	protected void setDrawModeOpaque()
//	{
//		gl.glBlendFunc(GL.GL_ONE, GL.GL_ZERO);
//		gl.glDisable(GL.GL_BLEND);
//		
//		currentBlendMode = BlendMode.Opaque;
//		setDrawMode(BlendMode.Opaque);
//	}
	
	protected void setDrawMode(BlendMode bmode)
	{
		if (bmode == BlendMode.Opaque)
		{
			gl.glBlendFunc(GL.GL_ONE, GL.GL_ZERO);
			gl.glDisable(GL.GL_BLEND);
		}
		else if (bmode == BlendMode.AlphaBlended)
		{
			gl.glEnable(GL.GL_BLEND);
			gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
		}
		
		currentBlendMode = bmode;
	}
	
	public void setMaskState(MaskState ms)
	{
		if (ms == MaskState.DrawNone)
		{
			gl.glColorMask(false, false, false, false);
			gl.glDepthMask(false);
		}
		else if (ms == MaskState.DrawDepthOnly)
		{
			gl.glColorMask(false, false, false, false);
			gl.glDepthMask(true);
		}
		else if (ms == MaskState.DrawColorOnly)
		{
			gl.glColorMask(true, true, true, true);
			gl.glDepthMask(false);
		}
		else if (ms == MaskState.DrawBoth)
		{
			gl.glColorMask(true, true, true, true);
			gl.glDepthMask(true);
		}
		
		currentMaskState = ms;
	}
	
	public void setCullState(CullState cs)
	{
		if (cs == CullState.CullNone)
		{
			gl.glDisable(GL.GL_CULL_FACE);
		}	
		else if (cs == CullState.CullFront)
		{
			gl.glEnable(GL.GL_CULL_FACE);		
			gl.glCullFace(GL.GL_FRONT);
		}		
		else if (cs == CullState.CullBack)
		{
			gl.glEnable(GL.GL_CULL_FACE);		
			gl.glCullFace(GL.GL_BACK);
		}		
		else if (cs == CullState.CullBoth)
		{
			gl.glEnable(GL.GL_CULL_FACE);		
			gl.glCullFace(GL.GL_FRONT_AND_BACK);
		}
		
		currentCullState = cs;
	}
	
	public void setDepthTest(boolean bTest)
	{
		if (bTest)
		{
			gl.glEnable(GL.GL_DEPTH_TEST);
		}
		else
		{
			gl.glDisable(GL.GL_DEPTH_TEST);
		}
		
		bDepthTest = bTest;
	}
}
