package org.processmining.visualisation3d.collision;

import java.util.Vector;

import org.processmining.visualisation3d.maths.JoglVectord3;

public class JoglAABB extends JoglCollisionShape
{
	private JoglVectord3 min;
	private JoglVectord3 max;
	private JoglVectord3 center;
	
	public JoglAABB(JoglVectord3 min, JoglVectord3 max)
	{
		this.min = min;
		this.max = max;
		this.center = JoglVectord3.Add(min, JoglVectord3.Multiply(JoglVectord3.Subtract(max, min), 0.5));
	}
	
	public JoglVectord3 getMin()
	{
		return min;
	}
	
	public JoglVectord3 getMax()
	{
		return max;
	}
	
	public JoglVectord3 getCenter()
	{
		return center;
	}
	
	public double getHeight()
	{
		return (max.getY() - min.getY());
	}
	
	@Override
	public boolean Intersects(JoglRay r)
	{
		double tmin = Double.NEGATIVE_INFINITY;
		double tmax = Double.POSITIVE_INFINITY;
	    
	    for (int i = 0; i < 3; i++)
	    {
	    	if (r.getDir().getValue(i) != 0.0)
	    	{
		        double tx1 = (min.getValue(i) - r.getOrigin().getValue(i))/r.getDir().getValue(i);
		        double tx2 = (max.getValue(i) - r.getOrigin().getValue(i))/r.getDir().getValue(i);
		 
		        tmin = Math.max(tmin, Math.min(tx1, tx2));
		        tmax = Math.min(tmax, Math.max(tx1, tx2));
	    	}
	    }
	 
	    return tmax >= tmin;
	}
	
	public double GetIntersectionDist(JoglRay r)
	{
		double tmin = Double.NEGATIVE_INFINITY;
		double tmax = Double.POSITIVE_INFINITY;
	    
	    for (int i = 0; i < 3; i++)
	    {
	    	if (r.getDir().getValue(i) != 0.0)
	    	{
		        double tx1 = (min.getValue(i) - r.getOrigin().getValue(i))/r.getDir().getValue(i);
		        double tx2 = (max.getValue(i) - r.getOrigin().getValue(i))/r.getDir().getValue(i);
		 
		        tmin = Math.max(tmin, Math.min(tx1, tx2));
		        tmax = Math.min(tmax, Math.max(tx1, tx2));
	    	}
	    }
	 
	    if (tmax >= tmin)
	    {
	    	if (tmin >= 0)
	    	{
	    		return tmin;
	    	}
	    	else
	    	{
	    		return tmax;
	    	}
	    }
	    return Double.POSITIVE_INFINITY;
	}
	
	public JoglVectord3 GetIntersectionPoint(JoglRay r)
	{
		double tmin = Double.NEGATIVE_INFINITY;
		double tmax = Double.POSITIVE_INFINITY;
	    
	    for (int i = 0; i < 3; i++)
	    {
	    	if (r.getDir().getValue(i) != 0.0)
	    	{
		        double tx1 = (min.getValue(i) - r.getOrigin().getValue(i))/r.getDir().getValue(i);
		        double tx2 = (max.getValue(i) - r.getOrigin().getValue(i))/r.getDir().getValue(i);
		 
		        tmin = Math.max(tmin, Math.min(tx1, tx2));
		        tmax = Math.min(tmax, Math.max(tx1, tx2));
	    	}
	    }
	 
	    if (tmax >= tmin)
	    {
	    	if (tmin >= 0)
	    	{
	    		return JoglVectord3.Add(r.getOrigin(), JoglVectord3.Multiply(r.getDir(), tmin));
	    	}
	    	else
	    	{
	    		return JoglVectord3.Add(r.getOrigin(), JoglVectord3.Multiply(r.getDir(), tmax));
	    	}
	    }
	    return null;
	}
	
	public Vector<JoglVectord3> getCorners()
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();
		
		lst.add(new JoglVectord3(min.getX(), min.getY(), min.getZ()));
		lst.add(new JoglVectord3(max.getX(), min.getY(), min.getZ()));
		lst.add(new JoglVectord3(min.getX(), max.getY(), min.getZ()));
		lst.add(new JoglVectord3(min.getX(), min.getY(), max.getZ()));
		
		lst.add(new JoglVectord3(max.getX(), max.getY(), min.getZ()));
		lst.add(new JoglVectord3(max.getX(), min.getY(), max.getZ()));
		lst.add(new JoglVectord3(min.getX(), max.getY(), max.getZ()));
		lst.add(new JoglVectord3(max.getX(), max.getY(), max.getZ()));
		
		return lst;
	}
	
	public JoglAABB Translate(JoglVectord3 offset)
	{
		return new JoglAABB(JoglVectord3.Add(min, offset), JoglVectord3.Add(max, offset));
	}
	
	public JoglAABB Scale(JoglVectord3 s)
	{
		return new JoglAABB(JoglVectord3.Multiply(min, s), JoglVectord3.Multiply(max, s));
	}

}
