package org.processmining.visualisation3d.collision;

import org.processmining.visualisation3d.maths.JoglVectord3;

public class JoglAABBFactory 
{
	private double minx = Double.POSITIVE_INFINITY;
	private double miny = Double.POSITIVE_INFINITY;
	private double minz = Double.POSITIVE_INFINITY;
	private double maxx = Double.NEGATIVE_INFINITY;
	private double maxy = Double.NEGATIVE_INFINITY;
	private double maxz = Double.NEGATIVE_INFINITY;
	
	private int totalPoints = 0;
	
	public JoglAABBFactory()
	{
		
	}
	
	public void reset()
	{
		minx = Double.POSITIVE_INFINITY;
		miny = Double.POSITIVE_INFINITY;
		minz = Double.POSITIVE_INFINITY;
		maxx = Double.NEGATIVE_INFINITY;
		maxy = Double.NEGATIVE_INFINITY;
		maxz = Double.NEGATIVE_INFINITY;
		totalPoints = 0;
	}
	
	public void addPoint(JoglVectord3 v)
	{
		minx = Math.min(minx, v.getX());
		miny = Math.min(miny, v.getY());
		minz = Math.min(minz, v.getZ());
		
		maxx = Math.max(maxx, v.getX());
		maxy = Math.max(maxy, v.getY());
		maxz = Math.max(maxz, v.getZ());
		
		totalPoints++;
	}
	
	public JoglAABB produce() throws CannotCreateBoundingBoxException
	{
		if (totalPoints > 0)
		{
			JoglVectord3 vmin = new JoglVectord3(minx, miny, minz);
			JoglVectord3 vmax = new JoglVectord3(maxx, maxy, maxz);
	
			return new JoglAABB(vmin, vmax);
		}
		throw new CannotCreateBoundingBoxException();
	}
}
