package org.processmining.visualisation3d.garbagecollection;

import java.util.Vector;

import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.gldatastructures.IGLResource;

public class GLGarbageCollector 
{
	private static Vector<IGLResource> uninited = new Vector<IGLResource>();
	private static Vector<IGLResource> tracked = new Vector<IGLResource>();
	private static Vector<IGLResource> garbage = new Vector<IGLResource>();
	
	public void Track(IGLResource d)
	{
		tracked.add(d);
	}
	
	public void Untrack(IGLResource d)
	{
		tracked.remove(d);
	}
	
	public void Uninit(GraphicsPipeline pipe)
	{	
		for (IGLResource r : new Vector<IGLResource>(tracked))
		{
			if (r.isLoaded())
			{
				r.unload(pipe);
			}
			uninited.add(r);
		}
	}
	
	public void Reinit(GraphicsPipeline pipe)
	{
		for (IGLResource r : new Vector<IGLResource>(uninited))
		{
			if (!r.isLoaded())
			{
				r.load(pipe);
			}
		}
		uninited.clear();
	}
	
	public void QueueForDisposal(IGLResource d)
	{
		if (tracked.contains(d))
		{
			tracked.remove(d);
		}
		garbage.add(d);
	}
	
	public void DisposeGarbage(GraphicsPipeline pipe)
	{
		for (IGLResource d : garbage)
		{
			//System.out.println("Disposing: " + d);
			d.unload(pipe);
		}
		garbage.clear();
	}
	
	public void Finalize()
	{	
		int refIndisposed = 0;
		for (IGLResource r : garbage)
		{
			if (r.isLoaded())
			{
				System.out.println("GLGC: " + r + " has not been properly disposed of!");
				refIndisposed++;
			}
		}
		System.out.println("GarbageReferences: " + garbage.size() + " (" + refIndisposed + " not disposed)");
		
		refIndisposed = 0;
		for (IGLResource r : tracked)
		{
			if (r.isLoaded())
			{
				System.out.println("GLGC: " + r + " has not been properly disposed of!");
				refIndisposed++;
			}
		}
		System.out.println("TrackedReferences: " + tracked.size() + " (" + refIndisposed + " not disposed)");
	}
}
