package org.processmining.visualisation3d.gldatastructures;

import java.nio.IntBuffer;

import org.processmining.visualisation3d.GraphicsPipeline;

import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;

public class JoglRenderBufferObject implements IJoglFrameBufferAttachment 
{
	private boolean bLoaded = false;
	private int renderbufferId = 0;
	private int width = 0;
	private int height = 0;
	private int bufferType = 0;
	
	private static int GenRenderbuffer(GL2 gl)
	{
		IntBuffer ib = IntBuffer.allocate(1);
		gl.glGenRenderbuffers(1, ib);
		
		return ib.get(0);
	}
	
	private static void DeleteRenderbuffer(GL2 gl, int id)
	{
		IntBuffer ib = IntBuffer.allocate(1);
		ib.put(id);
		ib.position(0);
		gl.glDeleteRenderbuffers(1, ib);
	}
	
	public int getRenderBufferId()
	{
		return renderbufferId;
	}
	
	public JoglRenderBufferObject(GraphicsPipeline pipe, int width, int height, int bufferType)
	{	
		this.width = width;
		this.height = height;
		this.bufferType = bufferType;
		
		load(pipe);
	}

	@Override
	public boolean isLoaded()
	{
		return bLoaded;
	}
	
	@Override
	public void load(GraphicsPipeline pipe) 
	{
		GL2 gl = pipe.gl;
		renderbufferId = GenRenderbuffer(gl);
		
		gl.glBindRenderbuffer(GL.GL_RENDERBUFFER, renderbufferId);
		gl.glRenderbufferStorage(GL.GL_RENDERBUFFER, bufferType, width, height);
		gl.glBindRenderbuffer(GL.GL_RENDERBUFFER, 0);

		if (pipe.gc != null)
		{
			pipe.gc.Track(this);
		}
		bLoaded = true;
	}
	
	public void attach(GraphicsPipeline pipe, int attachPoint)
	{
		pipe.gl.glBindRenderbuffer(GL.GL_RENDERBUFFER, getRenderBufferId());
		pipe.gl.glFramebufferRenderbuffer(GL.GL_FRAMEBUFFER, attachPoint, GL.GL_RENDERBUFFER, getRenderBufferId());
		pipe.gl.glBindRenderbuffer(GL.GL_RENDERBUFFER, 0);	
	}

	public void Dispose(GL2 gl)
	{
		GraphicsPipeline pipe = new GraphicsPipeline();
		pipe.gl = gl;
		unload(pipe);
	}

	@Override
	public void unload(GraphicsPipeline pipe) 
	{
		DeleteRenderbuffer(pipe.gl, renderbufferId);
		renderbufferId = 0;

		if (pipe.gc != null)
		{
			pipe.gc.Untrack(this);
		}
		bLoaded = false;
	}
	
	public void Bind(GL2 gl)
	{
		gl.glBindRenderbuffer(GL.GL_RENDERBUFFER, renderbufferId);
	}
	
	public void Unbind(GL2 gl)
	{
		gl.glBindRenderbuffer(GL.GL_RENDERBUFFER, 0);
	}

	public boolean isDisposed() 
	{
		return renderbufferId == 0;
	}
}
