package org.processmining.visualisation3d.gldatastructures;

import java.nio.IntBuffer;

import org.processmining.visualisation3d.GraphicsPipeline;

import com.jogamp.opengl.GL2;
import com.jogamp.opengl.GL3;

public class JoglTexSampler implements IGLResource
{
	private boolean bLoaded = false;
	private int samplerId = -1;

	private int texUnit = 1;
	private int minFilter = GL2.GL_LINEAR;
	private int magFilter = GL2.GL_LINEAR;
	private float anisoSamples = 1;
	
	private static int GenSampler(GL3 gl3)
	{
		IntBuffer ib = IntBuffer.allocate(1);
		gl3.glGenSamplers(1, ib);
		return ib.get(0);
	}
	
	private static void DeleteSampler(GL3 gl, int id)
	{
		IntBuffer ib = IntBuffer.allocate(1);
		ib.put(id);
		ib.position(0);
		gl.glDeleteSamplers(1, ib);
	}
	
	public JoglTexSampler(GraphicsPipeline pipe)
	{
		load(pipe);
	}

	@Override
	public boolean isLoaded()
	{
		return bLoaded;
	}
	
	@Override
	public void load(GraphicsPipeline pipe)
	{
		GL3 gl3 = pipe.gl.getGL3();
		samplerId = GenSampler(gl3);
		
		Bind(gl3, texUnit);
		setMinFilter(gl3, minFilter);
		setMagFilter(gl3, magFilter);
		setMaxAnisotropy(gl3, anisoSamples);
		Unbind(gl3, texUnit);
		
		if (pipe.gc != null)
		{
			pipe.gc.Track(this);
		}
		bLoaded = true;
	}
	
	@Override
	public void unload(GraphicsPipeline pipe)
	{
		GL3 gl3 = pipe.gl.getGL3();
		
		DeleteSampler(gl3, samplerId);
		samplerId = -1;
		
		if (pipe.gc != null)
		{
			pipe.gc.Untrack(this);
		}
		bLoaded = false;
	}
	
	public void setParameteri(GL3 gl3, int pName, int pVal)
	{
		gl3.glSamplerParameteri(samplerId, pName, pVal);	
	}
	
	public void setParameterf(GL3 gl3, int pName, float pVal)
	{
		gl3.glSamplerParameterf(samplerId, pName, pVal);	
	}
	
	public void setMinFilter(GL3 gl3, int pVal)
	{
		minFilter = pVal;
		setParameteri(gl3, GL2.GL_TEXTURE_MIN_FILTER, pVal);	
	}
	
	public void setMagFilter(GL3 gl3, int pVal)
	{
		magFilter = pVal;
		setParameteri(gl3, GL2.GL_TEXTURE_MAG_FILTER, pVal);	
	}
	
	public void setMaxAnisotropy(GL3 gl3, float pVal)
	{
		anisoSamples = pVal;
		setParameterf(gl3, GL2.GL_TEXTURE_MAX_ANISOTROPY_EXT, pVal);	
	}
	
	public void Bind(GL3 gl3, int texUnit)
	{
		this.texUnit = texUnit;
		gl3.glBindSampler(texUnit, samplerId);
	}
	
	public void Unbind(GL3 gl3, int texUnit)
	{
		gl3.glBindSampler(texUnit, 0);
	}
}
