package org.processmining.visualisation3d.gldatastructures;

import java.nio.ByteBuffer;

import org.processmining.visualisation3d.GraphicsPipeline;

import com.jogamp.common.nio.Buffers;
import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.util.texture.Texture;

public class JoglTextureBufferObject implements IJoglFrameBufferAttachment 
{
	private boolean bLoaded = false;
	private JoglTexture tex;
	
	public int getTextureId()
	{
		return tex.getTextureId();
		//return texId;
	}
	
	public JoglTexture getTexture()
	{
		return tex;	
	}
	
	public JoglTextureBufferObject(GraphicsPipeline pipe, int width, int height, int intFormat, int border, int pFormat, int pType)
	{	
		int texId = emptyTexture(pipe.gl, width, height, intFormat, border, pFormat, pType);		
		tex = new JoglTexture(pipe, new Texture(texId, GL.GL_TEXTURE_2D, width, height, width, height, false));
		
		bLoaded = true;
		if (pipe.gc != null)
		{
			pipe.gc.Track(this);
		}
	}

	@Override
	public boolean isLoaded()
	{
		return bLoaded;
	}
	
	@Override
	public void load(GraphicsPipeline pipe) 
	{
		if (tex != null && !tex.isLoaded())
		{
			tex.load(pipe);
		}

		if (pipe.gc != null)
		{
			pipe.gc.Track(this);
		}
		bLoaded = true;
	}

	@Override
	public void unload(GraphicsPipeline pipe) 
	{
		if (tex != null && tex.isLoaded())
		{
			tex.unload(pipe);
		}

		if (pipe.gc != null)
		{
			pipe.gc.Untrack(this);
		}
		bLoaded = false;
	}

    private int emptyTexture(GL2 gl, int w, int h, int intFormat, int border, int pFormat, int pType) 
    {  
    	// Create An Empty Texture
        // Create Storage Space For Texture Data (128x128x4)
        //ByteBuffer data = BufferUtil.newByteBuffer(w * h * 4); //Java 1.8
        ByteBuffer data = Buffers.newDirectByteBuffer(w * h * 4); 
        data.limit(data.capacity());

        int[] txtnumber = new int[1];
        gl.glGenTextures(1, txtnumber, 0);  // Create 1 Texture
        gl.glBindTexture(GL.GL_TEXTURE_2D, txtnumber[0]);  // Bind The Texture
        
        // Build Texture Using Information In data
        gl.glTexImage2D(GL.GL_TEXTURE_2D, 0, intFormat, w, h, border,
                pFormat, pType, data);    
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR);

        return txtnumber[0];  // Return The Texture ID
    }
	
	public void attach(GraphicsPipeline pipe, int attachPoint)
	{
		if (tex != null)
		{
			pipe.gl.glFramebufferTexture2D(GL.GL_FRAMEBUFFER, attachPoint, GL.GL_TEXTURE_2D, tex.getTextureId(), 0);
		}
	}

	public void Dispose(GraphicsPipeline pipe)
	{
		tex.Dispose(pipe);
		tex = null;
		
		if (pipe.gc != null)
		{
			pipe.gc.Untrack(this);
		}
		bLoaded = false;
	}
	
	public void Bind(GL2 gl)
	{
		//gl.glBindRenderbuffer(GL.GL_RENDERBUFFER, renderbufferId);
	}
	
	public void Unbind(GL2 gl)
	{
		//gl.glBindRenderbuffer(GL.GL_RENDERBUFFER, 0);
	}

	public boolean isDisposed() 
	{
		return tex == null;
	}
}
