package org.processmining.visualisation3d.gldatastructures;

import java.io.IOException;
import java.net.URL;

import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

import com.jogamp.opengl.GL2;
import com.jogamp.opengl.util.texture.Texture;
import com.jogamp.opengl.util.texture.TextureData;
import com.jogamp.opengl.util.texture.TextureIO;

public class JoglTextureDelayedLoad extends JoglTexture
{
	private String resLoc = "";
	private boolean bLoadedD = false;
	
	public JoglTextureDelayedLoad(JoglScene scene, String trl)
	{
		resLoc = trl;
	}

	@Override
	public void load(GraphicsPipeline pipe)
	{
		GL2 gl = pipe.gl;
		
		if (bLoadedD)
		{
			Load(pipe, resLoc, true);
//			if (tdata != null)
//			{
//				tex = TextureIO.newTexture(tdata);
//				tex.enable(gl);
//				tex.bind(gl);
//				tex.setTexParameteri(gl, GL.GL_TEXTURE_MIN_FILTER, minFilter);
//				tex.setTexParameteri(gl, GL.GL_TEXTURE_MAG_FILTER, magFilter);
//				tex.setTexParameteri(gl, GL.GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, anisoSamples);
//			}
		}
		if (pipe.gc != null)
		{
			pipe.gc.Track(this);
		}
		bLoaded = true;
	}

	@Override
	public void unload(GraphicsPipeline pipe)
	{
		if (bLoadedD)
		{
			if (tex != null)
			{
				try
				{
					//tdata = getTextureData(pipe.gl);
					tex.destroy(pipe.gl);
					tex = null;
				}
				catch (Exception ex)
				{
					System.out.println(ex.getMessage());
				}
			}
		}
		if (pipe.gc != null)
		{
			pipe.gc.Untrack(this);
		}
		bLoaded = false;
	}
	
	private void Load(GraphicsPipeline pipe, String texResLoc, boolean bMipMap)
	{
		GL2 gl = pipe.gl;
		
		System.out.println("Loading texture: " + texResLoc);
		URL fileUrl = this.getClass().getResource(texResLoc);

		String texFileType = DetermineFileTypeHint(texResLoc);
		
		try
		{
			TextureData td = TextureIO.newTextureData(gl.getGLProfile(), fileUrl, bMipMap, texFileType);
			Texture tx = TextureIO.newTexture(td);
		
			tex = tx;

			setDefaultFilterSettings(pipe, bMipMap);

			if (pipe.gc != null)
			{
				pipe.gc.Track(this);
			}
			bLoaded = true;
			bLoadedD = true;
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			//e.printStackTrace();
			System.out.println(e.getMessage());
		}
	}
	
	@Override
	public void Bind(GraphicsPipeline pipe)
	{
		if (!bLoadedD)
		{
			Load(pipe, resLoc, true);
		}
		tex.bind(pipe.gl);
	}

}
