package org.processmining.visualisation3d.gldatastructures;

import java.nio.Buffer;
import java.nio.ByteBuffer;
import java.nio.DoubleBuffer;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import java.nio.ShortBuffer;

import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.maths.JoglVectord4;

import com.jogamp.common.nio.Buffers;
import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;

public class JoglVertexArrayObjectBuffer implements IJoglVertexArrayObjectBuffer
{
	private boolean bLoaded = false;
    private int bufferHandle = 0;

    private int dataTypeSize = 1;
    private boolean bNormalize = true;

    private int pointerSize = 1;
    private int pointerType;

	private static int GenBuffer(GL2 gl)
	{
		IntBuffer ib = IntBuffer.allocate(1);
		gl.glGenBuffers(1, ib);
		
		return ib.get(0);
	}
	
	private static void DeleteBuffer(GL2 gl, int id)
	{
		IntBuffer ib = IntBuffer.allocate(1);
		ib.put(id);
		ib.position(0);
		gl.glDeleteBuffers(1, ib);
	}
	
    public JoglVertexArrayObjectBuffer(GraphicsPipeline pipe, Class<?> keyType, boolean bNorm)
    {
        bNormalize = bNorm;

        if (keyType == Float.class)
        {
            dataTypeSize = 4;
            pointerType = GL.GL_FLOAT;
            pointerSize = 1;
        }
        else if (keyType == Double.class)
        {
            dataTypeSize = 8;
            pointerType = GL2.GL_DOUBLE;
            pointerSize = 1;
        }
        else if (keyType == Byte.class)
        {
            dataTypeSize = 1;
            pointerType = GL.GL_BYTE;
            pointerSize = 1;
        }
        else if (keyType == Short.class)
        {
            dataTypeSize = 2;
            pointerType = GL.GL_SHORT;
            pointerSize = 1;
        }
        else if (keyType == Integer.class)
        {
            dataTypeSize = 4;
            pointerType = GL2.GL_INT;
            pointerSize = 1;
        }
        else if (keyType == JoglVectord2.class)
        {
            dataTypeSize = 8 * 2;
            pointerType = GL2.GL_DOUBLE;
            pointerSize = 2;
        }
        else if (keyType == JoglVectord3.class)
        {
            dataTypeSize = 8 * 3;
            pointerType = GL2.GL_DOUBLE;
            pointerSize = 3;
        }
        else if (keyType == JoglVectord4.class)
        {
            dataTypeSize = 8 * 4;
            pointerType = GL2.GL_DOUBLE;
            pointerSize = 4;
        }
        
        load(pipe);
    }
    
    @Override
    public boolean isLoaded()
    {
    	return bLoaded;
    }
    
    @Override
    public void load(GraphicsPipeline pipe)
    {
        bufferHandle = GenBuffer(pipe.gl);

        Bind(pipe);
        if (parray != null)
        {
        	if (parray instanceof int[])
        	{
        		IntBuffer b = Buffers.newDirectIntBuffer((int[])parray);
        		b.rewind();
                pipe.gl.glBufferData(GL.GL_ARRAY_BUFFER,
                    b.capacity() * (dataTypeSize / pointerSize),
                    b, GL.GL_STATIC_DRAW);
        	}
        	else if (parray instanceof short[])
        	{
        		ShortBuffer b = Buffers.newDirectShortBuffer((short[])parray);
        		b.rewind();
                pipe.gl.glBufferData(GL.GL_ARRAY_BUFFER,
                    b.capacity() * (dataTypeSize / pointerSize),
                    b, GL.GL_STATIC_DRAW);
        	}
        	else if (parray instanceof byte[])
        	{
        		ByteBuffer b = Buffers.newDirectByteBuffer((byte[])parray);
        		b.rewind();
                pipe.gl.glBufferData(GL.GL_ARRAY_BUFFER,
                    b.capacity() * (dataTypeSize / pointerSize),
                    b, GL.GL_STATIC_DRAW);
        	}
        	else if (parray instanceof float[])
        	{
        		FloatBuffer b = Buffers.newDirectFloatBuffer((float[])parray);
        		b.rewind();
                pipe.gl.glBufferData(GL.GL_ARRAY_BUFFER,
                    b.capacity() * (dataTypeSize / pointerSize),
                    b, GL.GL_STATIC_DRAW);
        	}
        	else if (parray instanceof double[])
        	{
        		DoubleBuffer b = Buffers.newDirectDoubleBuffer((double[])parray);
        		b.rewind();
                pipe.gl.glBufferData(GL.GL_ARRAY_BUFFER,
                    b.capacity() * (dataTypeSize / pointerSize),
                    b, GL.GL_STATIC_DRAW);
        	}
        }
        Unbind(pipe);

        if (pipe.gc != null)
        {
        	pipe.gc.Track(this);
        }
    	bLoaded = true;
    }
    
    @Override
    public void unload(GraphicsPipeline pipe)
    {
		DeleteBuffer(pipe.gl, bufferHandle);
		bufferHandle = 0;

		if (pipe.gc != null)
		{
			pipe.gc.Untrack(this);
		}
    	bLoaded = false;
    }

    Object parray = null;
    
    public void SetData(GL2 gl, Buffer b)
    {
    	if (b instanceof IntBuffer)
    	{
    		IntBuffer ib = (IntBuffer)b;
    		b.rewind();
    		int[] iarray = new int[b.capacity()];
    		ib.get(iarray);
    		parray = iarray;
    		b.rewind();
    	}
    	else if (b instanceof ShortBuffer)
    	{
    		ShortBuffer ib = (ShortBuffer)b;
    		b.rewind();
    		short[] iarray = new short[b.capacity()];
    		ib.get(iarray);
    		parray = iarray;
    		b.rewind();
    	}
    	else if (b instanceof ByteBuffer)
    	{
    		ByteBuffer ib = (ByteBuffer)b;
    		b.rewind();
    		byte[] iarray = new byte[b.capacity()];
    		ib.get(iarray);
    		parray = iarray;
    		b.rewind();
    	}
    	else if (b instanceof FloatBuffer)
    	{
    		FloatBuffer ib = (FloatBuffer)b;
    		b.rewind();
    		float[] iarray = new float[b.capacity()];
    		ib.get(iarray);
    		parray = iarray;
    		b.rewind();
    	}
    	else if (b instanceof DoubleBuffer)
    	{
    		DoubleBuffer ib = (DoubleBuffer)b;
    		b.rewind();
    		double[] iarray = new double[b.capacity()];
    		ib.get(iarray);
    		parray = iarray;
    		b.rewind();
    	}
    	
    	try
    	{
	        gl.glBufferData(GL.GL_ARRAY_BUFFER,
	            b.capacity() * (dataTypeSize / pointerSize),
	            b, GL.GL_STATIC_DRAW);
    	}
    	catch (Exception ex)
    	{
    		System.out.println(ex.getMessage());
    	}
    }
    
    public void UpdateData(GraphicsPipeline pipe, Buffer b)
    {
    	if (b instanceof IntBuffer)
    	{
    		IntBuffer ib = (IntBuffer)b;
    		b.rewind();
    		int[] iarray = new int[b.capacity()];
    		ib.get(iarray);
    		parray = iarray;
    		b.rewind();
    	}
    	else if (b instanceof ShortBuffer)
    	{
    		ShortBuffer ib = (ShortBuffer)b;
    		b.rewind();
    		short[] iarray = new short[b.capacity()];
    		ib.get(iarray);
    		parray = iarray;
    		b.rewind();
    	}
    	else if (b instanceof ByteBuffer)
    	{
    		ByteBuffer ib = (ByteBuffer)b;
    		b.rewind();
    		byte[] iarray = new byte[b.capacity()];
    		ib.get(iarray);
    		parray = iarray;
    		b.rewind();
    	}
    	else if (b instanceof FloatBuffer)
    	{
    		FloatBuffer ib = (FloatBuffer)b;
    		b.rewind();
    		float[] iarray = new float[b.capacity()];
    		ib.get(iarray);
    		parray = iarray;
    		b.rewind();
    	}
    	else if (b instanceof DoubleBuffer)
    	{
    		DoubleBuffer ib = (DoubleBuffer)b;
    		b.rewind();
    		double[] iarray = new double[b.capacity()];
    		ib.get(iarray);
    		parray = iarray;
    		b.rewind();
    	}

    	try
    	{
    		pipe.gl.glBufferSubData(GL.GL_ARRAY_BUFFER, 0, b.capacity() * (dataTypeSize / pointerSize), b);
    	}
		catch (Exception ex)
		{
			System.out.println(ex.getMessage());
		}
    }

    @Override
    public void Bind(GraphicsPipeline pipe)
    {
        pipe.gl.glBindBuffer(GL.GL_ARRAY_BUFFER, bufferHandle);
    }

    @Override
    public void Unbind(GraphicsPipeline pipe)
    {
        pipe.gl.glBindBuffer(GL.GL_ARRAY_BUFFER, 0);
    }

    @Override
    public void Attach(GraphicsPipeline pipe, int attributePos)
    {
        pipe.gl.glEnableVertexAttribArray(attributePos);
        Bind(pipe);
        pipe.gl.glVertexAttribPointer(attributePos, pointerSize, pointerType, bNormalize, dataTypeSize, 0);
    }

	public void Dispose(GraphicsPipeline pipe) 
	{
        unload(pipe);
	}

	public boolean isDisposed() 
	{
		return bufferHandle == 0;
	}
}
