package org.processmining.visualisation3d.gldatastructures;

import java.nio.Buffer;
import java.nio.ByteBuffer;
import java.nio.IntBuffer;
import java.nio.ShortBuffer;

import org.processmining.visualisation3d.GraphicsPipeline;

import com.jogamp.common.nio.Buffers;
import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;

public class JoglVertexArrayObjectIndexBuffer implements IJoglVertexArrayObjectIndexBuffer
{
	private boolean bLoaded = false;
    private int bufferHandle = 0;

    private int dataTypeSize = 1;

    private int indexType;
    private int indexCount = 0;

	private static int GenIndexbuffer(GL2 gl)
	{
		IntBuffer ib = IntBuffer.allocate(1);
		gl.glGenBuffers(1, ib);
		
		return ib.get(0);
	}
	
	private static void DeleteIndexbuffer(GL2 gl, int id)
	{
		IntBuffer ib = IntBuffer.allocate(1);
		ib.put(id);
		ib.position(0);
		gl.glDeleteBuffers(1, ib);
	}
	
    public JoglVertexArrayObjectIndexBuffer(GraphicsPipeline pipe, Class<?> keyType)
    {
        if (keyType == Byte.class)
        {
            dataTypeSize = 1;
            indexType = GL.GL_UNSIGNED_BYTE;
        }
        else if (keyType == Short.class)
        {
            dataTypeSize = 2;
            indexType = GL.GL_UNSIGNED_SHORT;
        }
        else if (keyType == Integer.class)
        {
            dataTypeSize = 4;
            indexType = GL2.GL_UNSIGNED_INT;
        }
        
        load(pipe);
    }
    
    @Override
    public boolean isLoaded()
    {
    	return bLoaded;
    }
    
    @Override
    public void load(GraphicsPipeline pipe)
    {
        bufferHandle = GenIndexbuffer(pipe.gl);
        indexCount = 0;

        Bind(pipe);
        if (parray != null)
        {
        	if (parray instanceof int[])
        	{
        		IntBuffer b = Buffers.newDirectIntBuffer((int[])parray);
        		b.rewind();
            	indexCount = b.capacity();
                pipe.gl.glBufferData(GL.GL_ELEMENT_ARRAY_BUFFER,
                    b.capacity() * dataTypeSize,
                    b, GL.GL_STATIC_DRAW);
        	}
        	else if (parray instanceof short[])
        	{
        		ShortBuffer b = Buffers.newDirectShortBuffer((short[])parray);
        		b.rewind();
            	indexCount = b.capacity();
                pipe.gl.glBufferData(GL.GL_ELEMENT_ARRAY_BUFFER,
                    b.capacity() * dataTypeSize,
                    b, GL.GL_STATIC_DRAW);
        	}
        	else if (parray instanceof byte[])
        	{
        		ByteBuffer b = Buffers.newDirectByteBuffer((byte[])parray);
        		b.rewind();
            	indexCount = b.capacity();
                pipe.gl.glBufferData(GL.GL_ELEMENT_ARRAY_BUFFER,
                    b.capacity() * dataTypeSize,
                    b, GL.GL_STATIC_DRAW);
        	}
        }
        Unbind(pipe);

        if (pipe.gc != null)
        {
        	pipe.gc.Track(this);
        }
    	bLoaded = true;
    }
    
    @Override
    public void unload(GraphicsPipeline pipe)
    {
		DeleteIndexbuffer(pipe.gl, bufferHandle);
		bufferHandle = 0;

		if (pipe.gc != null)
		{
			pipe.gc.Untrack(this);
		}
    	bLoaded = false;
    }

    Object parray = null;
    
    public void SetData(GL2 gl, Buffer b)
    {
    	if (b instanceof IntBuffer)
    	{
    		IntBuffer ib = (IntBuffer)b;
    		b.rewind();
    		int[] iarray = new int[b.capacity()];
    		ib.get(iarray);
    		parray = iarray;
    		b.rewind();
    	}
    	else if (b instanceof ShortBuffer)
    	{
    		ShortBuffer ib = (ShortBuffer)b;
    		b.rewind();
    		short[] iarray = new short[b.capacity()];
    		ib.get(iarray);
    		parray = iarray;
    		b.rewind();
    	}
    	else if (b instanceof ByteBuffer)
    	{
    		ByteBuffer ib = (ByteBuffer)b;
    		b.rewind();
    		byte[] iarray = new byte[b.capacity()];
    		ib.get(iarray);
    		parray = iarray;
    		b.rewind();
    	}
    	
    	indexCount = b.capacity();
    	
    	try
    	{
	        gl.glBufferData(GL.GL_ELEMENT_ARRAY_BUFFER,
	            b.capacity() * dataTypeSize,
	            b, GL.GL_STATIC_DRAW);
    	}
    	catch (Exception ex)
    	{
    		System.out.println(ex.getMessage());
    	}
    }

    @Override
    public void Bind(GraphicsPipeline pipe)
    {
        pipe.gl.glBindBuffer(GL.GL_ELEMENT_ARRAY_BUFFER, bufferHandle);
    }

    @Override
    public void Unbind(GraphicsPipeline pipe)
    {
        pipe.gl.glBindBuffer(GL.GL_ELEMENT_ARRAY_BUFFER, 0);
    }

    @Override
	public void Attach(GraphicsPipeline pipe) 
	{
        Bind(pipe);
	}

	public void Draw(GL2 gl, int mode) 
	{
		if (bLoaded)
		{
			gl.glDrawElements(mode, indexCount, indexType, 0);
		}
	}

	public void Draw(GL2 gl, int mode, int off, int count) 
	{
		if (bLoaded)
		{
			if ((off + count) <= indexCount)
			{
				gl.glDrawElements(mode, count, indexType, off * dataTypeSize);
			}
			else
			{
				System.err.println("Trying to draw out of bounds of indexbuffer.");
			}
		}
	}

	public void Dispose(GL2 gl) 
	{
        GraphicsPipeline pipe = new GraphicsPipeline();
        pipe.gl = gl;
        
        unload(pipe);
	}

	public boolean isDisposed() 
	{
		return bufferHandle == 0;
	}
}
