package org.processmining.visualisation3d.graphicsdatastructures;

import org.processmining.visualisation3d.collision.JoglRay;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;

public class JoglCameraLensOrthographic extends JoglCameraLens 
{
	float zoom = 1.0f;
	float scaleW = 500.0f * (float)aspect;
	float scaleH = 500.0f;
	float scaleD = 5000.0f;
	//private float fieldOfView = 45.0f;
	//private double tanFoV = Math.tan(45.0 * 0.5 * 0.0174532925);
//	public double getFieldOfView()
//	{
//		return fieldOfView;
//	}
//	
//	public double getTanFoV()
//	{
//		return tanFoV;
//	}

	public float getZoom()
	{
		return zoom;
	}
	
	public double getScaleD()
	{
		return scaleD;
	}
	
	public double getScaleW()
	{
		return scaleW;
	}
	
	public double getScaleH()
	{
		return scaleH;
	}
	
	public JoglCameraLensOrthographic(double aspect)
	{
		this.aspect = aspect;
		scaleW = scaleW * (float)aspect;
		projMatrix = createOrthoProjectionMatrix(-scaleW, scaleW, scaleH, -scaleH, -scaleD, scaleD);
	}
	
	public void setZoom(float z)
	{
		zoom = z;
		//projMatrix = createOrthoProjectionMatrix(-scaleW * z, scaleW * z, scaleH * z, -scaleH * z, -scaleD * z, scaleD * z);
	}
	
	public JoglMatrixd4x4 createOrthoProjectionMatrix(float left, float right, float top, float bottom, float near, float far)
	{
		JoglMatrixd4x4 m = JoglMatrixd4x4.CreateIdentity();
				
		m.SetValue(0, 0, 2.0f / (right - left));
		m.SetValue(0, 1, 0.0f);
		m.SetValue(0, 2, 0.0f);
		m.SetValue(0, 3, 0.0f);
				
		m.SetValue(1, 0, 0.0f);
		m.SetValue(1, 1, 2.0f / (top - bottom));
		m.SetValue(1, 2, 0.0f);
		m.SetValue(1, 3, 0.0f);
				
		m.SetValue(2, 0, 0.0f);
		m.SetValue(2, 1, 0.0f);
		m.SetValue(2, 2, -2.0f / (far - near));
		m.SetValue(2, 3, 0.0f);
				
		m.SetValue(3, 0, -(right + left  ) / (right - left  ));
		m.SetValue(3, 1, -(top   + bottom) / (top   - bottom));
		m.SetValue(3, 2, -(far   + near  ) / (far   - near  ));
		m.SetValue(3, 3, 1.0f);
				
		return m;
	}

//	public JoglCameraLensOrthographic(double fov, double aspect)
//	{
//		this.aspect = aspect;
//		fieldOfView = (float)fov;
//		tanFoV = Math.tan(fieldOfView * 0.5 * 0.0174532925);
//		
//		projMatrix = GetProjectionMatrixOpt(aspect);
//	}
//
//	public JoglCameraLensOrthographic(double fov, double aspect, double nDist, double fDist)
//	{
//		this.aspect = aspect;
//		fieldOfView = (float)fov;
//		tanFoV = Math.tan(fieldOfView * 0.5 * 0.0174532925);
//		nearDist = (float)nDist;
//		farDist = (float)fDist;
//		
//		projMatrix = GetProjectionMatrixOpt(aspect);		
//	}
//
//	public JoglCameraLensOrthographic(double fov, double aspect, double nDist, double fDist, double l, double r, double b, double t)
//	{
//		this.aspect = aspect;
//		fieldOfView = (float)fov;
//		tanFoV = Math.tan(fieldOfView * 0.5 * 0.0174532925);
//		nearDist = (float)nDist;
//		farDist = (float)fDist;
//		
//		projMatrix = GetProjectionMatrixOpt(aspect, l, r, b, t);	
//	}
	
	@Override
	public JoglVectord3 ProjectToScreen(double x, double y, double z, double aspect)
	{
		JoglVectord3 v = new JoglVectord3((x * zoom) / scaleW, (y * zoom) / scaleH, (z * zoom) / scaleD);
		return v;
	}
//	
//	private JoglMatrixd4x4 GetProjectionMatrixOpt(double aspect) 
//	{
//		//adapted from http://nehe.gamedev.net/article/replacement_for_gluperspective/21002/
//		double fH = tanFoV * nearDist;
//		double fW = fH * aspect;
//
//		JoglMatrixd4x4 mat = JoglMatrixd4x4.CreateFrustum(nearDist, farDist, -fW, fW, -fH, fH);
//		return mat;
//	}
//
//	private JoglMatrixd4x4 GetProjectionMatrixOpt(double aspect, double left, double right, double top, double bottom) 
//	{
//		//adapted from http://nehe.gamedev.net/article/replacement_for_gluperspective/21002/
//		double fH = tanFoV * nearDist;
//		double fW = fH * aspect;
//
//		double w = fW * 2.0;
//		double h = fH * 2.0;
//		
//		double l = -fW;
//		//double r = fW;
//		double b = -fH;
//		//double t = fH;
//		
//		JoglMatrixd4x4 mat = JoglMatrixd4x4.CreateFrustum(nearDist, farDist, l + (w * left), l + (w * right), b + (h * bottom), b + (h * top));
//		return mat;
//	}
//
    
    @Override
    public JoglRay GetRay(JoglCamera cam, JoglVectord2 p)
    {
        //float aspect = (float)viewWidth / (float)viewHeight;

        float px = (float)p.getX();
        float py = (float)p.getY();
        
        float ndcx = ((px * 2.0f) - 1.0f);
        float ndcy = ((py * 2.0f) - 1.0f);

        //ray position in camera space
        //JoglVectord3 pixelPos = GetRayHelper(aspect, px, py);

        //transform ray position into world space
        //JoglVectord3 rayForward = JoglVectord3.Multiply(cam.GetLookDir(), pixelPos.getZ());
        JoglVectord3 rayLeft = JoglVectord3.Multiply(cam.GetLeftDir(), ndcx * -scaleW);
        JoglVectord3 rayUp = JoglVectord3.Multiply(cam.GetUpDir(), ndcy * -scaleH);
        
        JoglVectord3 rayPos = JoglVectord3.Add(rayLeft, rayUp);

		JoglRay r = new JoglRay(JoglVectord3.Add(cam.GetPosition(), rayPos), cam.GetLookDir());
		return r;
    }
}
