package org.processmining.visualisation3d.graphicsdatastructures;

import java.nio.Buffer;
import java.nio.IntBuffer;
import java.nio.ShortBuffer;

import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.gldatastructures.JoglVertexArrayObjectIndexBuffer;

import com.jogamp.common.nio.Buffers;

public class JoglMeshIndexBuffer implements IJoglMeshIndexBuffer
{
	public static JoglMeshIndexBuffer GenerateIndicesTri(int numVerts)
	{      
        //System.out.println("Model vert num: " + mesh.getNumVerts());
        
        if (numVerts < 65536)
        {
	        ShortBuffer indata = Buffers.newDirectShortBuffer(numVerts);
	        for (int i = 0; i < numVerts; i++)
	        {
	            indata.put((short)i);
	        }
	        indata.rewind();
	        
	        return new JoglMeshIndexBuffer(indata);
        }
        else
        {
	        IntBuffer indata = Buffers.newDirectIntBuffer(numVerts);
	        for (int i = 0; i < numVerts; i++)
	        {
	            indata.put(i);
	        }
	        indata.rewind();
	        
	        return new JoglMeshIndexBuffer(indata);
        }
	}
	
	public static JoglMeshIndexBuffer GenerateIndicesQuad(int numVerts)
	{
        if (numVerts < 65536)
        {
	        ShortBuffer indata = Buffers.newDirectShortBuffer((numVerts / 4) * 6);
	        for (int i = 0; i < numVerts; i = i + 4)
	        {
	            indata.put((short)i);
	            indata.put((short)(i+1));
	            indata.put((short)(i+2));
	            
	            indata.put((short)(i+2));
	            indata.put((short)(i+3));
	            indata.put((short)i);
	        }
	        indata.rewind();
	        
	        return new JoglMeshIndexBuffer(indata);
        }
        else
        {
	        IntBuffer indata = Buffers.newDirectIntBuffer((numVerts / 4) * 6);
	        for (int i = 0; i < numVerts; i = i + 4)
	        {
	            indata.put(i);
	            indata.put(i+1);
	            indata.put(i+2);
	            
	            indata.put(i+2);
	            indata.put(i+3);
	            indata.put(i);
	        }
	        indata.rewind();
	        
	        return new JoglMeshIndexBuffer(indata);
        }
	}
	
	public static JoglMeshIndexBuffer GenerateIndicesLineStrip(int numVerts)
	{
        if (numVerts < 65536)
        {
	        ShortBuffer indata = Buffers.newDirectShortBuffer((numVerts - 1) * 2);
	        for (int i = 0; i < (numVerts - 1); i++)
	        {
	            indata.put((short)i);
	            indata.put((short)(i+1));
	        }
	        indata.rewind();
	        
	        return new JoglMeshIndexBuffer(indata);
        }
        else
        {
	        IntBuffer indata = Buffers.newDirectIntBuffer((numVerts - 1) * 2);
	        for (int i = 0; i < (numVerts - 1); i++)
	        {
	            indata.put(i);
	            indata.put(i+1);
	        }
	        indata.rewind();
	        
	        return new JoglMeshIndexBuffer(indata);
        }
	}
	
	private Buffer buf = null;
	
	public JoglMeshIndexBuffer()
	{
		
	}
	
	public JoglMeshIndexBuffer(Buffer b)
	{
		buf = b;
	}
	
	@Override
    public JoglVertexArrayObjectIndexBuffer getVAOBuffer(GraphicsPipeline pipe)
    {
		JoglVertexArrayObjectIndexBuffer indbuf = null;
		
        if (buf instanceof ShortBuffer)
        {
	        indbuf = new JoglVertexArrayObjectIndexBuffer(pipe, Short.class);
        }
        else
        {
	        indbuf = new JoglVertexArrayObjectIndexBuffer(pipe, Integer.class);
        }
        indbuf.Bind(pipe);
        indbuf.SetData(pipe.gl, buf);
        indbuf.Unbind(pipe);
        
    	return indbuf;
    }
}
