package org.processmining.visualisation3d.interactions;

import java.awt.event.MouseEvent;

import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.input.DefaultKeyBindings;
import org.processmining.visualisation3d.input.JoglInputKeyEvent;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JICameraFree extends JICameraBase
{
	private boolean bForwardPressed = false;
	private boolean bBackwardPressed = false;
	private boolean bLeftPressed = false;
	private boolean bRightPressed = false;
	private boolean bForward2Pressed = false;
	private boolean bBackward2Pressed = false;
	private boolean bLeft2Pressed = false;
	private boolean bRight2Pressed = false;
	private boolean bTurnLeftPressed = false;
	private boolean bTurnRightPressed = false;
	private boolean bShiftPressed = false;
	
	private double camDx = 0.0;
	private double camDy = 0.0;

	private double cPitch = Math.PI * 0.5;
	private double cYaw = Math.PI;
 
    private boolean bMouseView = false;
    private JoglVectord2 lastMousePos;
    
    public double moveStepDefault = 2;
    public double moveStepRun = 10;
    public double turnStepDefault = 0.1;
    
    //private double camTurnFactorX = 0.005;
    //private double camTurnFactorY = 0.005;
    private double camTurnFactorX = 5.55;
    private double camTurnFactorY = 5.55;
	
    public JICameraFree(JoglCamera c)
    {
		super(c);
    }
    
	public void setPitchYawRoll(double p, double y, double r)
	{
		cPitch = p;
		cYaw = y;
		
		cam.SetPitchYawRoll(cPitch, cYaw, 0);
	}
	
	private void RotateCamera(double x, double y)
	{
		if (x != 0)
		{
			cYaw = cYaw + x;
		}
		
		if (y != 0)
		{
			cPitch = cPitch + y;
			cPitch = Math.min(cPitch, Math.PI * 0.5);
			cPitch = Math.max(cPitch, Math.PI * -0.5);
		}  	
		cam.SetPitchYawRoll(cPitch, cYaw, 0);	
		
		//System.out.println("Cam: " + cPitch + "/" + cYaw);
	}
	
	private boolean IsKeyPressed()
	{
		if (bForwardPressed || bBackwardPressed || bLeftPressed || bRightPressed)
		{
	    	return true;
		}
		if (bForward2Pressed || bBackward2Pressed || bLeft2Pressed || bRight2Pressed)
		{
	    	return true;
		}
		if (bTurnLeftPressed || bTurnRightPressed)
		{
			return true;
		}
		if (bMouseView)
		{
			return true;
		}
		return false;
	}

	@Override
	public boolean ShouldAnimate()
	{
		return IsKeyPressed();
	}

	@Override
	public void UpdateScene(JoglScene scene)
	{
		//System.out.println("Camera update scene");
	    double moveStep = moveStepDefault;
	    double turnStep = turnStepDefault;

        if (bShiftPressed)
        {
        	moveStep = moveStepRun;
        }
        
		if (bTurnLeftPressed)
		{
			camDx = camDx - turnStep;
		}
		if (bTurnRightPressed)
		{
			camDx = camDx + turnStep;
		}
		
		if (camDx != 0 || camDy != 0)
		{

	        RotateCamera(camDx, camDy);
	        
	        camDx = 0.0;
	        camDy = 0.0;
		}
		
		if (bForwardPressed || bForward2Pressed)
		{
	    	JoglVectord3 ldir = JoglVectord3.Normal(cam.GetLookDir()); 
	    	
	    	cam.SetPosition(JoglVectord3.Add(cam.GetPosition(), JoglVectord3.Multiply(ldir, moveStep)));
	    	cam.SetLookAt(JoglVectord3.Add(cam.GetLookAt(), JoglVectord3.Multiply(ldir, moveStep)));
		}
		if (bBackwardPressed || bBackward2Pressed)
		{
        	JoglVectord3 ldir = JoglVectord3.Normal(cam.GetLookDir());  	
        	
        	cam.SetPosition(JoglVectord3.Add(cam.GetPosition(), JoglVectord3.Multiply(ldir, -moveStep)));
        	cam.SetLookAt(JoglVectord3.Add(cam.GetLookAt(), JoglVectord3.Multiply(ldir, -moveStep)));
		}
		if (bLeftPressed || bLeft2Pressed)
		{
        	JoglVectord3 leftdir = cam.GetLeftDir();
        	
        	cam.SetPosition(JoglVectord3.Add(cam.GetPosition(), JoglVectord3.Multiply(leftdir, moveStep)));
        	cam.SetLookAt(JoglVectord3.Add(cam.GetLookAt(), JoglVectord3.Multiply(leftdir, moveStep)));			
		}
		if (bRightPressed || bRight2Pressed)
		{
        	JoglVectord3 leftdir = cam.GetLeftDir();
        	
        	cam.SetPosition(JoglVectord3.Add(cam.GetPosition(), JoglVectord3.Multiply(leftdir, -moveStep)));
        	cam.SetLookAt(JoglVectord3.Add(cam.GetLookAt(), JoglVectord3.Multiply(leftdir, -moveStep)));			
		}	
	}

	//KeyListener method implementations
	@Override
	public boolean keyPressed(JoglInputKeyEvent evt) 
	{
        //System.out.println(evt);
        
        if (evt.getKeyCode() == DefaultKeyBindings.Camera_Down) //s key
        {
        	bBackwardPressed = true;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_Down2) //down key
        {
        	bBackward2Pressed = true;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_Up) //w key
        {
        	bForwardPressed = true;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_Up2) //up key
        {
        	bForward2Pressed = true;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_Left2) //left key
        {
        	bLeft2Pressed = true;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_Left) //a key
        {
        	bLeftPressed = true;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_Right2) //right key
        {
        	bRight2Pressed = true;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_Right) //d key
        {
        	bRightPressed = true;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_RotateLeft) //q key
        {
        	//RotateCamera(0.1, 0);
        	bTurnLeftPressed = true;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_RotateRight) //e key
        {
        	//RotateCamera(-0.1, 0);
        	bTurnRightPressed = true;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_Run) //shift key
        {
        	bShiftPressed = true;
        }
        
        return false;
	}

	@Override
	public boolean keyReleased(JoglInputKeyEvent evt) 
	{
        //System.out.println(evt); 
        if (evt.getKeyCode() == DefaultKeyBindings.Camera_Down) //s key
        {
        	bBackwardPressed = false;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_Down2) //down key
        {
        	bBackward2Pressed = false;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_Up2) //up key
        {
        	bForward2Pressed = false;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_Up) //w key
        {
        	bForwardPressed = false;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_Left2) //left key
        {
        	bLeft2Pressed = false;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_Left) //a key
        {
        	bLeftPressed = false;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_Right2) //right key
        {
        	bRight2Pressed = false;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_Right) //d key
        {
        	bRightPressed = false;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_RotateLeft) //q key
        {
        	bTurnLeftPressed = false;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_RotateRight) //e key
        {
        	bTurnRightPressed = false;
        }
        else if (evt.getKeyCode() == DefaultKeyBindings.Camera_Run) //shift key
        {
        	bShiftPressed = false;
        }
        
        return false;
	}
	
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
      //System.out.println("mouse pressed: " + e.getButton());

      if (e.getButton() == MouseEvent.BUTTON3) 
      {
    	  lastMousePos = e.getPoint();
    	  bMouseView = true;
//    	  if (bAnimPaused)
//    	  {
//    		  anim.resume();
//    	  }
      }
      
      return false;
    }

	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
        if (e.getButton() == MouseEvent.BUTTON3) 
        {
      	  bMouseView = false;
        }
        
        return false;
    }

	@Override
    public void mouseDragged(JoglInputMouseEvent e) 
    {
    	if (bMouseView)
    	{
	        JoglVectord2 newPos = e.getPoint();
	        double dx = newPos.getX() - lastMousePos.getX();
	        double dy = newPos.getY() - lastMousePos.getY();

	        double aspect = cam.getCameraLens().getAspect();
			camDx = camDx + (dx * camTurnFactorX * aspect);
	        camDy = camDy + (dy * camTurnFactorY);
	        
    	}
        lastMousePos = e.getPoint();
    }

}
