package org.processmining.visualisation3d.materials;

import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.settings.JoglGraphicsSettings;

public class MaterialGrid extends MaterialGridBase
{
	private boolean oldDepthTest = true;
	private MaskState oldMaskState = MaskState.DrawBoth;
	
	private JoglShader shad = null;
	private boolean bSwitchedOffDepth = false;
	
	public MaterialGrid()
	{
	}

	@Override
	public void Apply(GraphicsPipeline pipe)
	{
		super.Apply(pipe);
		
		InitShaders(pipe, true);
		
		if (shad != null)
		{
			shad.Bind(pipe);
		}
	}

	@Override
	public void Unapply(GraphicsPipeline pipe)
	{
		super.Unapply(pipe);

		if (shad != null)
		{
			shad.Unbind(pipe);
		}
		
		if (bSwitchedOffDepth)
		{
			setDepthTest(oldDepthTest);
			setMaskState(oldMaskState);
			bSwitchedOffDepth = false;
		}
	}
	
	@Override
	public void setDepthState(boolean bDepthOff)
	{
		if (bDepthOff && !bSwitchedOffDepth)
		{
			oldDepthTest = bDepthTest;
			oldMaskState = currentMaskState;
			
			setDepthTest(false);
			setMaskState(MaskState.DrawColorOnly);
			bSwitchedOffDepth = true;
		}
	}
	
	@Override
	public void DrawColor(JoglVectord4 gridColor)
	{
		setColor(gridColor);		
	}

	private void InitShaders(GraphicsPipeline pipe, boolean bMasked) 
	{
		if (JoglGraphicsSettings.SupportsShaderObjects(pipe))
		{
			if (shad == null)
			{
				if (JoglResourceManager.HasResource("GridShader"))
				{
					shad = (JoglShader)JoglResourceManager.GetResource("GridShader");
				}
				else
				{
					shad = new JoglShader("/org/processmining/processprofiler3d/compviz/jogl/resources", "gridShaderVS", "gridShaderFS");
					shad.Create(pipe);
					
					JoglResourceManager.AddResource("GridShader", shad);
				}
			}
		}
	}
}
