package org.processmining.visualisation3d.maths;

import java.util.Vector;

public class JoglCatmullRomSpline 
{
    private Vector<JoglVectord3> points = new Vector<JoglVectord3>();
    private JoglMatrixd m = new JoglMatrixd(4, 4);

    public JoglCatmullRomSpline(Vector<JoglVectord3> pts)
    {
        m.SetValue(0, 0, 0);
        m.SetValue(0, 1, 2);
        m.SetValue(0, 2, 0);
        m.SetValue(0, 3, 0);

        m.SetValue(1, 0, -1);
        m.SetValue(1, 1, 0);
        m.SetValue(1, 2, 1);
        m.SetValue(1, 3, 0);

        m.SetValue(2, 0, 2);
        m.SetValue(2, 1, -5);
        m.SetValue(2, 2, 4);
        m.SetValue(2, 3, -1);

        m.SetValue(3, 0, -1);
        m.SetValue(3, 1, 3);
        m.SetValue(3, 2, -3);
        m.SetValue(3, 3, 1);
        
        points = pts;
    }

    public JoglVectord3 GetPos(double dp)
    {
        int p1 = ((int)Math.floor(dp)) - 1;
        int p2 = ((int)Math.floor(dp));
        int p3 = ((int)Math.ceil(dp));
        int p4 = ((int)Math.ceil(dp)) + 1;
        double dps = dp - p2;

        p1 = (p1 + points.size()) % points.size();
        p2 = (p2 + points.size()) % points.size();
        p3 = (p3 + points.size()) % points.size();
        p4 = (p4 + points.size()) % points.size();

        JoglMatrixd o = new JoglMatrixd(1, 4);
        o.SetValue(0, 0, 1.0);
        o.SetValue(0, 1, dps);
        o.SetValue(0, 2, dps * dps);
        o.SetValue(0, 3, dps * dps * dps);

        JoglMatrixd n = new JoglMatrixd(4, 3);
        n.SetValue(0, 0, points.get(p1).getX());
        n.SetValue(0, 1, points.get(p1).getY());
        n.SetValue(0, 2, points.get(p1).getZ());

        n.SetValue(1, 0, points.get(p2).getX());
        n.SetValue(1, 1, points.get(p2).getY());
        n.SetValue(1, 2, points.get(p2).getZ());

        n.SetValue(2, 0, points.get(p3).getX());
        n.SetValue(2, 1, points.get(p3).getY());
        n.SetValue(2, 2, points.get(p3).getZ());

        n.SetValue(3, 0, points.get(p4).getX());
        n.SetValue(3, 1, points.get(p4).getY());
        n.SetValue(3, 2, points.get(p4).getZ());

        JoglMatrixd res = JoglMatrixd.Multiply(JoglMatrixd.Multiply(o, m), n);
        res = JoglMatrixd.Multiply(res, 0.5);

        return new JoglVectord3(res.GetValue(0, 0), res.GetValue(0, 1), res.GetValue(0, 2));
    }

}
