package org.processmining.visualisation3d.maths;

public class JoglVectord2 
{
	public static final JoglVectord2 ZERO = new JoglVectord2(0,0);
	
	private double[] data;
	
	public JoglVectord2()
	{
		data = new double[2];
	}
	
	public JoglVectord2(double x, double y)
	{
		data = new double[2];
		
		data[0] = x;
		data[1] = y;
	}
	
	public double[] getData()
	{
		return data;
	}
	
	public void setData(double[] d)
	{
		data = d;
	}
	
	public double getValue(int i)
	{
		return data[i];
	}
	
	public double getX()
	{
		return data[0];
	}
	
	public double getY()
	{
		return data[1];
	}
	
	public void setValue(int i, double v)
	{
		data[i] = v;
	}
	
	public void setX(double v)
	{
		data[0] = v;
	}
	
	public void setY(double v)
	{
		data[1] = v;
	}
	
	public double getMagnitude()
    {
        double temp = 0;

        for (int i = 0; i < 2; i++)
        {
            temp += (data[i] * data[i]);
        }
        return Math.sqrt(temp);
    }
	
	public double getMagnitudeSquared()
    {
        double temp = 0;

        for (int i = 0; i < 2; i++)
        {
            temp += (data[i] * data[i]);
        }
        return temp;
    }
	
	public static double Distance(JoglVectord2 v1, JoglVectord2 v2)
	{
		return JoglVectord2.Subtract(v1, v2).getMagnitude();
	}
	
	public static JoglVectord2 Normal(JoglVectord2 v)
    {
        return Divide(v , v.getMagnitude());
    }
	
	public static JoglVectord2 Add(JoglVectord2 v1, JoglVectord2 v2)
	{
		return new JoglVectord2(v1.getValue(0) + v2.getValue(0), v1.getValue(1) + v2.getValue(1));
	}
	
	public static JoglVectord2 Subtract(JoglVectord2 v1, JoglVectord2 v2)
	{
		return new JoglVectord2(v1.getValue(0) - v2.getValue(0), v1.getValue(1) - v2.getValue(1));
	}
	
	public static JoglVectord2 Multiply(JoglVectord2 v1, double d)
	{
		return new JoglVectord2(v1.getValue(0) * d, v1.getValue(1) * d);
	}
	
	public static JoglVectord2 Divide(JoglVectord2 v1, double d)
	{
		return new JoglVectord2(v1.getValue(0) / d, v1.getValue(1) / d);
	}
	
	public static double Dot(JoglVectord2 v1, JoglVectord2 v2)
    {
        double dotProduct = 0;
        for (int i = 0; i < 2; i++)
        {
            dotProduct += (v1.getValue(i) * v2.getValue(i));
        }
        return dotProduct;
    }
	
	public static JoglVectord2 Blend(JoglVectord2 col1, JoglVectord2 col2, float mixVal)
	{
		return JoglVectord2.Add(JoglVectord2.Multiply(col1, mixVal), JoglVectord2.Multiply(col2, 1.0f - mixVal));
	}

}
