package org.processmining.visualisation3d.maths;

public class JoglVectord3 
{
	public static final JoglVectord3 ZERO = new JoglVectord3(0,0,0);
	
	private double[] data;
	
	public JoglVectord3()
	{
		data = new double[3];
	}
	
	public JoglVectord3(double x, double y, double z)
	{
		data = new double[3];
		
		data[0] = x;
		data[1] = y;
		data[2] = z;
	}
	
	public double[] getData()
	{
		return data;
	}
	
	public void setData(double[] d)
	{
		data = d;
	}
	
	public double getValue(int i)
	{
		return data[i];
	}
	
	public double getX()
	{
		return data[0];
	}
	
	public double getY()
	{
		return data[1];
	}
	
	public double getZ()
	{
		return data[2];
	}
	
	public void setValue(int i, double v)
	{
		data[i] = v;
	}
	
	public void setX(double v)
	{
		data[0] = v;
	}
	
	public void setY(double v)
	{
		data[1] = v;
	}
	
	public void setZ(double v)
	{
		data[2] = v;
	}
	
	public double getMagnitude()
    {
        double temp = 0;

        for (int i = 0; i < 3; i++)
        {
            temp += (data[i] * data[i]);
        }
        return Math.sqrt(temp);
    }
	
	public double getMagnitudeSquared()
    {
        double temp = 0;

        for (int i = 0; i < 3; i++)
        {
            temp += (data[i] * data[i]);
        }
        return temp;
    }
	
	public static double Distance(JoglVectord3 v1, JoglVectord3 v2)
	{
		return JoglVectord3.Subtract(v1, v2).getMagnitude();
	}
	
	public static JoglVectord3 Normal(JoglVectord3 v)
    {
        return Divide(v , v.getMagnitude());
    }
	
	public static JoglVectord3 Add(JoglVectord3 v1, JoglVectord3 v2)
	{
		return new JoglVectord3(v1.getValue(0) + v2.getValue(0), v1.getValue(1) + v2.getValue(1), v1.getValue(2) + v2.getValue(2));
	}
	
	public static JoglVectord3 Subtract(JoglVectord3 v1, JoglVectord3 v2)
	{
		return new JoglVectord3(v1.getValue(0) - v2.getValue(0), v1.getValue(1) - v2.getValue(1), v1.getValue(2) - v2.getValue(2));
	}
	
	public static JoglVectord3 Multiply(JoglVectord3 v1, double d)
	{
		return new JoglVectord3(v1.getValue(0) * d, v1.getValue(1) * d, v1.getValue(2) * d);
	}
	
	public static JoglVectord3 Multiply(JoglVectord3 v1, JoglVectord3 d)
	{
		return new JoglVectord3(v1.getValue(0) * d.getX(), v1.getValue(1) * d.getY(), v1.getValue(2) * d.getZ());
	}
	
	public static JoglVectord3 Divide(JoglVectord3 v1, double d)
	{
		return new JoglVectord3(v1.getValue(0) / d, v1.getValue(1) / d, v1.getValue(2) / d);
	}
	
	public static JoglVectord3 Divide(JoglVectord3 v1, JoglVectord3 d)
	{
		return new JoglVectord3(v1.getValue(0) / d.getX(), v1.getValue(1) / d.getY(), v1.getValue(2) / d.getZ());
	}
	
	public static double Dot(JoglVectord3 v1, JoglVectord3 v2)
    {
        double dotProduct = 0;
        for (int i = 0; i < 3; i++)
        {
            dotProduct += (v1.getValue(i) * v2.getValue(i));
        }
        return dotProduct;
    }
	
	public static JoglVectord3 Cross(JoglVectord3 v1, JoglVectord3 v2)
    {
        return new JoglVectord3(((v1.getValue(1) * v2.getValue(2)) - (v1.getValue(2) * v2.getValue(1))), -((v1.getValue(0) * v2.getValue(2)) - (v1.getValue(2) * v2.getValue(0))), ((v1.getValue(0) * v2.getValue(1)) - (v1.getValue(1) * v2.getValue(0))));
    }
	
	public static JoglVectord3 Blend(JoglVectord3 col1, JoglVectord3 col2, float mixVal)
	{
		return JoglVectord3.Add(JoglVectord3.Multiply(col1, 1.0f - mixVal), JoglVectord3.Multiply(col2, mixVal));
	}
}
