package org.processmining.visualisation3d.maths;

import java.nio.DoubleBuffer;
import java.util.Vector;

import com.jogamp.common.nio.Buffers;

public class JoglVectord3Buffer 
{
	protected DoubleBuffer vertsd;
	
	public JoglVectord3Buffer()
	{
		vertsd = Buffers.newDirectDoubleBuffer(3);
		vertsd.limit(0);
	}
	
	public JoglVectord3Buffer(int vCapacity)
	{
		vertsd = Buffers.newDirectDoubleBuffer(vCapacity * 3);
		vertsd.limit(0);
	}
	
	public int getNum()
	{
		return (vertsd.limit() / 3);
	}
	
	public double[] getArray()
	{
		double[] dst = new double[vertsd.limit()];
		vertsd.position(0);
		vertsd.get(dst, 0, vertsd.limit());
		
		return dst;
	}
	
	public DoubleBuffer getNativeBuffer()
	{
		return vertsd;
	}
	
	public void position(int pos)
	{
		vertsd.position(pos * 3);
	}
	
	public void set(double[] vlist)
	{
		int newCap = vlist.length;

		if (newCap > vertsd.capacity())
		{
			vertsd = Buffers.newDirectDoubleBuffer(vlist);
		}
		else
		{
			vertsd.position(0);
			vertsd.limit(newCap);
			vertsd.put(vlist);
		}
	}
	
	public void set(Vector<JoglVectord3> vlist)
	{
		int newCap = vlist.size() * 3;
		if (newCap > vertsd.capacity())
		{
			vertsd = Buffers.newDirectDoubleBuffer(newCap);
			
			for (int i = 0; i < vlist.size(); i++)
			{
				JoglVectord3 v = vlist.get(i);
				vertsd.put(v.getData());
			}
		}
		else
		{
			vertsd.position(0);
			vertsd.limit(newCap);
			for (int i = 0; i < vlist.size(); i++)
			{
				JoglVectord3 v = vlist.get(i);
				vertsd.put(v.getData());
			}
		}
	}
	
	public void set(double[] vlist, JoglVectord3 offset)
	{
		int newCap = vlist.length;

		if (newCap > vertsd.capacity())
		{
			vertsd = Buffers.newDirectDoubleBuffer(newCap);
			for (int i = 0; i < vlist.length; i++)
			{
				double d = vlist[i] + offset.getValue(i % 3);
				vertsd.put(d);
			}
		}
		else
		{
			vertsd.position(0);
			vertsd.limit(newCap);
			//vertsd.put(vlist);
			for (int i = 0; i < vlist.length; i++)
			{
				double d = vlist[i] + offset.getValue(i % 3);
				vertsd.put(d);
			}
		}
	}
	
	public void add(double[] vlist)
	{
		int oldCap = vertsd.limit();
		int newCap = oldCap + vlist.length;

		if (newCap > vertsd.capacity())
		{
			DoubleBuffer nvertsd = Buffers.newDirectDoubleBuffer(newCap);
			
			//copy old verts
			vertsd.limit(oldCap);
			vertsd.position(0);
			nvertsd.put(vertsd);
	
			nvertsd.put(vlist);
			
			vertsd = nvertsd;
		}
		else
		{
			vertsd.limit(newCap);
			vertsd.put(vlist);
		}
	}
	
	public void add(Vector<JoglVectord3> vlist)
	{
		int oldCap = vertsd.limit();
		int newCap = oldCap + (vlist.size() * 3);

		if (newCap > vertsd.capacity())
		{
			DoubleBuffer nvertsd = Buffers.newDirectDoubleBuffer(newCap);
			
			//copy old verts
			vertsd.limit(oldCap);
			vertsd.position(0);
			nvertsd.put(vertsd);
			
			for (int i = 0; i < vlist.size(); i++)
			{
				JoglVectord3 v = vlist.get(i);
				nvertsd.put(v.getData());
			}
			
			vertsd = nvertsd;
		}
		else
		{	
			vertsd.limit(newCap);
			for (int i = 0; i < vlist.size(); i++)
			{
				JoglVectord3 v = vlist.get(i);
				vertsd.put(v.getData());
			}
		}
	}
	
	public void add(DoubleBuffer vlist)
	{
		int oldCap = vertsd.limit();
		int newCap = oldCap + vlist.limit();

		if (newCap > vertsd.capacity())
		{
			DoubleBuffer nvertsd = Buffers.newDirectDoubleBuffer(newCap);
			
			//copy old verts
			vertsd.limit(oldCap);
			vertsd.position(0);
			nvertsd.put(vertsd);
	
			vlist.position(0);
			nvertsd.put(vlist);
			
			vertsd = nvertsd;
		}
		else
		{
			vlist.position(0);
			vertsd.limit(newCap);
			vertsd.put(vlist);
		}
	}
	
//	public void add(JoglMesh m)
//	{
//		DoubleBuffer vlist = m.getVertsBuffer();
//		int oldCap = vertsd.limit();
//		int newCap = oldCap + vlist.limit();
//
//		if (newCap > vertsd.capacity())
//		{
//			DoubleBuffer nvertsd = Buffers.newDirectDoubleBuffer(newCap);
//			
//			//copy old verts
//			vertsd.limit(oldCap);
//			vertsd.position(0);
//			nvertsd.put(vertsd);
//	
//			vlist.position(0);
//			nvertsd.put(vlist);
//			
//			vertsd = nvertsd;
//		}
//		else
//		{
//			vlist.position(0);
//			vertsd.limit(newCap);
//			vertsd.put(vlist);
//		}
//	}
	
	public void add(Vector<JoglVectord3> vlist, JoglVectord3 offset)
	{
		int oldCap = vertsd.limit();
		int numCap = oldCap + (vlist.size() * 3);

		if (numCap > vertsd.capacity())
		{
			DoubleBuffer nvertsd = Buffers.newDirectDoubleBuffer(numCap);
			
			//copy old verts
			vertsd.limit(oldCap);
			vertsd.position(0);
			nvertsd.put(vertsd);
			
			for (int i = 0; i < vlist.size(); i++)
			{
				JoglVectord3 v = JoglVectord3.Add(vlist.get(i), offset);
				nvertsd.put(v.getData());
			}
			
			vertsd = nvertsd;
		}
		else
		{
			vertsd.limit(numCap);
			for (int i = 0; i < vlist.size(); i++)
			{
				JoglVectord3 v = JoglVectord3.Add(vlist.get(i), offset);
				vertsd.put(v.getData());
			}
		}
	}
	
	public void add(double[] vlist, JoglVectord3 offset)
	{
		int oldCap = vertsd.limit();
		int newCap = oldCap + vlist.length;

		if (newCap > vertsd.capacity())
		{
			DoubleBuffer nvertsd = Buffers.newDirectDoubleBuffer(newCap);
			
			//copy old verts
			vertsd.limit(oldCap);
			vertsd.position(0);
			nvertsd.put(vertsd);
			
			double doffx = offset.getX();
			double doffy = offset.getY();
			double doffz = offset.getZ();
			
			for (int i = 0; i < vlist.length; i = i + 3)
			{
				nvertsd.put(vlist[i] + doffx);
				nvertsd.put(vlist[i+1] + doffy);
				nvertsd.put(vlist[i+2] + doffz);
			}
			
			vertsd = nvertsd;
		}
		else
		{
			double doffx = offset.getX();
			double doffy = offset.getY();
			double doffz = offset.getZ();
			
			vertsd.limit(newCap);	
			for (int i = 0; i < vlist.length; i = i + 3)
			{
				vertsd.put(vlist[i] + doffx);
				vertsd.put(vlist[i+1] + doffy);
				vertsd.put(vlist[i+2] + doffz);
			}
		}
	}
	
	public void add(DoubleBuffer vlist, JoglVectord3 offset)
	{
		int oldCap = vertsd.limit();
		int newCap = oldCap + vlist.limit();

		if (newCap > vertsd.capacity())
		{
			DoubleBuffer nvertsd = Buffers.newDirectDoubleBuffer(newCap);
			
			//copy old verts
			vertsd.limit(oldCap);
			vertsd.position(0);
			nvertsd.put(vertsd);
			
			double doffx = offset.getX();
			double doffy = offset.getY();
			double doffz = offset.getZ();
			
			vlist.position(0);
			for (int i = 0; i < vlist.limit(); i = i + 3)
			{
				nvertsd.put(vlist.get() + doffx);
				nvertsd.put(vlist.get() + doffy);
				nvertsd.put(vlist.get() + doffz);
			}
			
			vertsd = nvertsd;
		}
		else
		{
			double doffx = offset.getX();
			double doffy = offset.getY();
			double doffz = offset.getZ();

			vlist.position(0);
			vertsd.limit(newCap);
			for (int i = 0; i < vlist.limit(); i = i + 3)
			{
				vertsd.put(vlist.get() + doffx);
				vertsd.put(vlist.get() + doffy);
				vertsd.put(vlist.get() + doffz);
			}
		}
	}
	
//	public void add(JoglMesh m, JoglVectord3 offset)
//	{
//		DoubleBuffer vlist = m.getVertsBuffer();
//		int oldCap = vertsd.limit();
//		int newCap = oldCap + vlist.limit();
//
//		if (newCap > vertsd.capacity())
//		{
//			DoubleBuffer nvertsd = Buffers.newDirectDoubleBuffer(newCap);
//			
//			//copy old verts
//			vertsd.limit(oldCap);
//			vertsd.position(0);
//			nvertsd.put(vertsd);
//			
//			double doffx = offset.getX();
//			double doffy = offset.getY();
//			double doffz = offset.getZ();
//			
//			vlist.position(0);
//			for (int i = 0; i < vlist.limit(); i = i + 3)
//			{
//				nvertsd.put(vlist.get() + doffx);
//				nvertsd.put(vlist.get() + doffy);
//				nvertsd.put(vlist.get() + doffz);
//			}
//			
//			vertsd = nvertsd;
//		}
//		else
//		{
//			double doffx = offset.getX();
//			double doffy = offset.getY();
//			double doffz = offset.getZ();
//
//			vlist.position(0);
//			vertsd.limit(newCap);
//			for (int i = 0; i < vlist.limit(); i = i + 3)
//			{
//				vertsd.put(vlist.get() + doffx);
//				vertsd.put(vlist.get() + doffy);
//				vertsd.put(vlist.get() + doffz);
//			}
//		}
//	}
	
	public void add(DoubleBuffer vlist, JoglMatrixd4x4 offset)
	{
		int oldCap = vertsd.limit();
		int newCap = oldCap + vlist.limit();

		if (newCap > vertsd.capacity())
		{
			DoubleBuffer nvertsd = Buffers.newDirectDoubleBuffer(newCap);
			
			//copy old verts
			vertsd.limit(oldCap);
			vertsd.position(0);
			nvertsd.put(vertsd);
			
			//offset.Transformv3(v)
			
//			double doffx = offset.getX();
//			double doffy = offset.getY();
//			double doffz = offset.getZ();
			
			vlist.position(0);
			for (int i = 0; i < vlist.limit(); i = i + 3)
			{
				JoglVectord3 vp = new JoglVectord3(vlist.get(), vlist.get(), vlist.get());
				vp = offset.Transformv3(vp);
//				nvertsd.put(vlist.get() + doffx);
//				nvertsd.put(vlist.get() + doffy);
//				nvertsd.put(vlist.get() + doffz);
				nvertsd.put(vp.getX());
				nvertsd.put(vp.getY());
				nvertsd.put(vp.getZ());
			}
			
			vertsd = nvertsd;
		}
		else
		{
//			double doffx = offset.getX();
//			double doffy = offset.getY();
//			double doffz = offset.getZ();

			vlist.position(0);
			vertsd.limit(newCap);
			for (int i = 0; i < vlist.limit(); i = i + 3)
			{
				JoglVectord3 vp = new JoglVectord3(vlist.get(), vlist.get(), vlist.get());
				vp = offset.Transformv3(vp);
//				vertsd.put(vlist.get() + doffx);
//				vertsd.put(vlist.get() + doffy);
//				vertsd.put(vlist.get() + doffz);
				vertsd.put(vp.getX());
				vertsd.put(vp.getY());
				vertsd.put(vp.getZ());
			}
		}
	}
	
//	public void add(JoglMesh m, JoglMatrixd4x4 offset)
//	{
//		DoubleBuffer vlist = m.getVertsBuffer();
//		int oldCap = vertsd.limit();
//		int newCap = oldCap + vlist.limit();
//
//		if (newCap > vertsd.capacity())
//		{
//			DoubleBuffer nvertsd = Buffers.newDirectDoubleBuffer(newCap);
//			
//			//copy old verts
//			vertsd.limit(oldCap);
//			vertsd.position(0);
//			nvertsd.put(vertsd);
//			
//			//offset.Transformv3(v)
//			
////			double doffx = offset.getX();
////			double doffy = offset.getY();
////			double doffz = offset.getZ();
//			
//			vlist.position(0);
//			for (int i = 0; i < vlist.limit(); i = i + 3)
//			{
//				JoglVectord3 vp = new JoglVectord3(vlist.get(), vlist.get(), vlist.get());
//				vp = offset.Transformv3(vp);
////				nvertsd.put(vlist.get() + doffx);
////				nvertsd.put(vlist.get() + doffy);
////				nvertsd.put(vlist.get() + doffz);
//				nvertsd.put(vp.getX());
//				nvertsd.put(vp.getY());
//				nvertsd.put(vp.getZ());
//			}
//			
//			vertsd = nvertsd;
//		}
//		else
//		{
////			double doffx = offset.getX();
////			double doffy = offset.getY();
////			double doffz = offset.getZ();
//
//			vlist.position(0);
//			vertsd.limit(newCap);
//			for (int i = 0; i < vlist.limit(); i = i + 3)
//			{
//				JoglVectord3 vp = new JoglVectord3(vlist.get(), vlist.get(), vlist.get());
//				vp = offset.Transformv3(vp);
////				vertsd.put(vlist.get() + doffx);
////				vertsd.put(vlist.get() + doffy);
////				vertsd.put(vlist.get() + doffz);
//				vertsd.put(vp.getX());
//				vertsd.put(vp.getY());
//				vertsd.put(vp.getZ());
//			}
//		}
//	}
	
//	public void add(JoglMesh m, JoglVectord3 offset, int vstart, int vend)
//	{
//		DoubleBuffer vlist = m.getVertsBuffer();
//		int oldCap = vertsd.limit();
//		int newCap = oldCap + (vend - vstart);
//
//		if (newCap > vertsd.capacity())
//		{
//			DoubleBuffer nvertsd = Buffers.newDirectDoubleBuffer(newCap);
//			
//			//copy old verts
//			vertsd.limit(oldCap);
//			vertsd.position(0);
//			nvertsd.put(vertsd);
//			
//			double doffx = offset.getX();
//			double doffy = offset.getY();
//			double doffz = offset.getZ();
//			
//			vlist.position(vstart);
//			for (int i = vstart; i < vend; i = i + 3)
//			{
//				nvertsd.put(vlist.get() + doffx);
//				nvertsd.put(vlist.get() + doffy);
//				nvertsd.put(vlist.get() + doffz);
//			}
//			
//			vertsd = nvertsd;
//		}
//		else
//		{
//			double doffx = offset.getX();
//			double doffy = offset.getY();
//			double doffz = offset.getZ();
//
//			vlist.position(vstart);
//			vertsd.limit(newCap);
//			for (int i = vstart; i < vend; i = i + 3)
//			{
//				vertsd.put(vlist.get() + doffx);
//				vertsd.put(vlist.get() + doffy);
//				vertsd.put(vlist.get() + doffz);
//			}
//		}
//	}
	
	public void add(DoubleBuffer vlist, JoglVectord3 offset, int vstart, int vend)
	{
		int oldCap = vertsd.limit();
		int newCap = oldCap + (vend - vstart);

		if (newCap > vertsd.capacity())
		{
			DoubleBuffer nvertsd = Buffers.newDirectDoubleBuffer(newCap);
			
			//copy old verts
			vertsd.limit(oldCap);
			vertsd.position(0);
			nvertsd.put(vertsd);
			
			double doffx = offset.getX();
			double doffy = offset.getY();
			double doffz = offset.getZ();
			
			vlist.position(vstart);
			for (int i = vstart; i < vend; i = i + 3)
			{
				nvertsd.put(vlist.get() + doffx);
				nvertsd.put(vlist.get() + doffy);
				nvertsd.put(vlist.get() + doffz);
			}
			
			vertsd = nvertsd;
		}
		else
		{
			double doffx = offset.getX();
			double doffy = offset.getY();
			double doffz = offset.getZ();

			vlist.position(vstart);
			vertsd.limit(newCap);
			for (int i = vstart; i < vend; i = i + 3)
			{
				vertsd.put(vlist.get() + doffx);
				vertsd.put(vlist.get() + doffy);
				vertsd.put(vlist.get() + doffz);
			}
		}
	}
	
//	public void drawNextVertex3d(GL2 gl)
//	{
//		gl.glVertex3d(vertsd.get(), vertsd.get(), vertsd.get());
//	}
//	
//	public void drawNextColor3d(GL2 gl)
//	{
//		gl.glColor3d(vertsd.get(), vertsd.get(), vertsd.get());
//	}
//	
//	public void drawNextNormal3d(GL2 gl)
//	{
//		gl.glNormal3d(vertsd.get(), vertsd.get(), vertsd.get());
//	}

}