package org.processmining.visualisation3d.maths;

public class JoglVectord4 
{
	private double[] data;
	
	public JoglVectord4()
	{
		data = new double[4];
	}
	
	public JoglVectord4(double x, double y, double z, double w)
	{
		data = new double[4];
		
		data[0] = x;
		data[1] = y;
		data[2] = z;
		data[3] = w;
	}
	
	public double[] getData()
	{
		return data;
	}
	
	public void setData(double[] d)
	{
		data = d;
	}
	
	public double getValue(int i)
	{
		return data[i];
	}
	
	public double getX()
	{
		return data[0];
	}
	
	public double getY()
	{
		return data[1];
	}
	
	public double getZ()
	{
		return data[2];
	}
	
	public double getW()
	{
		return data[3];
	}
	
	public void setValue(int i, double v)
	{
		data[i] = v;
	}
	
	public void setX(double v)
	{
		data[0] = v;
	}
	
	public void setY(double v)
	{
		data[1] = v;
	}
	
	public void setZ(double v)
	{
		data[2] = v;
	}
	
	public double getMagnitude()
    {
        double temp = 0;

        for (int i = 0; i < 4; i++)
        {
            temp += (data[i] * data[i]);
        }
        return Math.sqrt(temp);
    }
	
	public static JoglVectord4 Normal(JoglVectord4 v)
    {
        return Divide(v , v.getMagnitude());
    }
	
	public static JoglVectord4 Add(JoglVectord4 v1, JoglVectord4 v2)
	{
		return new JoglVectord4(v1.getValue(0) + v2.getValue(0), v1.getValue(1) + v2.getValue(1), v1.getValue(2) + v2.getValue(2), v1.getValue(3) + v2.getValue(3));
	}
	
	public static JoglVectord4 Subtract(JoglVectord4 v1, JoglVectord4 v2)
	{
		return new JoglVectord4(v1.getValue(0) - v2.getValue(0), v1.getValue(1) - v2.getValue(1), v1.getValue(2) - v2.getValue(2), v1.getValue(3) - v2.getValue(3));
	}
	
	public static JoglVectord4 Multiply(JoglVectord4 v1, double d)
	{
		return new JoglVectord4(v1.getValue(0) * d, v1.getValue(1) * d, v1.getValue(2) * d, v1.getValue(3) * d);
	}
	
	public static JoglVectord4 Divide(JoglVectord4 v1, double d)
	{
		return new JoglVectord4(v1.getValue(0) / d, v1.getValue(1) / d, v1.getValue(2) / d, v1.getValue(3) / d);
	}
	
	public static double Dot(JoglVectord4 v1, JoglVectord4 v2)
    {
        double dotProduct = 0;
        for (int i = 0; i < 4; i++)
        {
            dotProduct += (v1.getValue(i) * v2.getValue(i));
        }
        return dotProduct;
    }
	
	public static JoglVectord4 Blend(JoglVectord4 col1, JoglVectord4 col2, float mixVal)
	{
		return JoglVectord4.Add(JoglVectord4.Multiply(col1, mixVal), JoglVectord4.Multiply(col2, 1.0f - mixVal));
	}

}
