package org.processmining.visualisation3d.maths;

import java.nio.DoubleBuffer;
import java.util.Vector;

import com.jogamp.common.nio.Buffers;

public class JoglVectord4Buffer 
{
	protected DoubleBuffer vertsd;
	
	public JoglVectord4Buffer()
	{
		vertsd = Buffers.newDirectDoubleBuffer(4);
		vertsd.limit(0);
	}
	
	public JoglVectord4Buffer(int vCapacity)
	{
		vertsd = Buffers.newDirectDoubleBuffer(vCapacity * 4);
		vertsd.limit(0);
	}
	
	public int getNum()
	{
		return (vertsd.limit() / 4);
	}
	
	public double[] getArray()
	{
		double[] dst = new double[vertsd.limit()];
		vertsd.position(0);
		vertsd.get(dst, 0, vertsd.limit());
		
		return dst;
	}
	
	public DoubleBuffer getNativeBuffer()
	{
		return vertsd;
	}
	
	public void position(int pos)
	{
		vertsd.position(pos * 4);
	}
	
	public void set(double[] vlist)
	{
		int newCap = vlist.length;

		if (newCap > vertsd.capacity())
		{
			vertsd = Buffers.newDirectDoubleBuffer(vlist);
		}
		else
		{
			vertsd.position(0);
			vertsd.limit(newCap);
			vertsd.put(vlist);
		}
	}
	
	public void set3(Vector<JoglVectord3> vlist)
	{
		int newCap = vlist.size() * 4;
		if (newCap > vertsd.capacity())
		{
			vertsd = Buffers.newDirectDoubleBuffer(newCap);
			
			for (int i = 0; i < vlist.size(); i++)
			{
				JoglVectord3 v3 = vlist.get(i);
				//JoglVectord4 v = new JoglVectord4(v3.getX(), v3.getY(), v3.getZ(), 1.0);
				double[] v = new double[]{v3.getX(), v3.getY(), v3.getZ(), 1.0};
				vertsd.put(v);
			}
		}
		else
		{
			vertsd.position(0);
			vertsd.limit(newCap);
			for (int i = 0; i < vlist.size(); i++)
			{
				JoglVectord3 v3 = vlist.get(i);
				//JoglVectord4 v = new JoglVectord4(v3.getX(), v3.getY(), v3.getZ(), 1.0);
				double[] v = new double[]{v3.getX(), v3.getY(), v3.getZ(), 1.0};
				vertsd.put(v);
			}
		}
	}
	
	public void set(Vector<JoglVectord4> vlist)
	{
		int newCap = vlist.size() * 4;
		if (newCap > vertsd.capacity())
		{
			vertsd = Buffers.newDirectDoubleBuffer(newCap);
			
			for (int i = 0; i < vlist.size(); i++)
			{
				JoglVectord4 v = vlist.get(i);
				vertsd.put(v.getData());
			}
		}
		else
		{
			vertsd.position(0);
			vertsd.limit(newCap);
			for (int i = 0; i < vlist.size(); i++)
			{
				JoglVectord4 v = vlist.get(i);
				vertsd.put(v.getData());
			}
		}
	}
	
	public void add(double[] vlist)
	{
		int oldCap = vertsd.limit();
		int newCap = oldCap + vlist.length;

		if (newCap > vertsd.capacity())
		{
			DoubleBuffer nvertsd = Buffers.newDirectDoubleBuffer(newCap);
			
			//copy old verts
			vertsd.limit(oldCap);
			vertsd.position(0);
			nvertsd.put(vertsd);
	
			nvertsd.put(vlist);
			
			vertsd = nvertsd;
		}
		else
		{
			vertsd.limit(newCap);
			vertsd.put(vlist);
		}
	}
	
	public void add3(Vector<JoglVectord3> vlist)
	{
		int oldCap = vertsd.limit();
		int newCap = oldCap + (vlist.size() * 4);

		if (newCap > vertsd.capacity())
		{
			DoubleBuffer nvertsd = Buffers.newDirectDoubleBuffer(newCap);
			
			//copy old verts
			vertsd.limit(oldCap);
			vertsd.position(0);
			nvertsd.put(vertsd);
			
			for (int i = 0; i < vlist.size(); i++)
			{
				JoglVectord3 v3 = vlist.get(i);
				//JoglVectord4 v = new JoglVectord4(v3.getX(), v3.getY(), v3.getZ(), 1.0);
				double[] v = new double[]{v3.getX(), v3.getY(), v3.getZ(), 1.0};
				nvertsd.put(v);
			}
			
			vertsd = nvertsd;
		}
		else
		{	
			vertsd.limit(newCap);
			for (int i = 0; i < vlist.size(); i++)
			{
				JoglVectord3 v3 = vlist.get(i);
				//JoglVectord4 v = new JoglVectord4(v3.getX(), v3.getY(), v3.getZ(), 1.0);
				double[] v = new double[]{v3.getX(), v3.getY(), v3.getZ(), 1.0};
				vertsd.put(v);
			}
		}
	}
	
	public void add(Vector<JoglVectord4> vlist)
	{
		int oldCap = vertsd.limit();
		int newCap = oldCap + (vlist.size() * 4);

		if (newCap > vertsd.capacity())
		{
			DoubleBuffer nvertsd = Buffers.newDirectDoubleBuffer(newCap);
			
			//copy old verts
			vertsd.limit(oldCap);
			vertsd.position(0);
			nvertsd.put(vertsd);
			
			for (int i = 0; i < vlist.size(); i++)
			{
				JoglVectord4 v = vlist.get(i);
				nvertsd.put(v.getData());
			}
			
			vertsd = nvertsd;
		}
		else
		{	
			vertsd.limit(newCap);
			for (int i = 0; i < vlist.size(); i++)
			{
				JoglVectord4 v = vlist.get(i);
				vertsd.put(v.getData());
			}
		}
	}
	
	public void add(DoubleBuffer vlist)
	{
		int oldCap = vertsd.limit();
		int newCap = oldCap + vlist.limit();

		if (newCap > vertsd.capacity())
		{
			DoubleBuffer nvertsd = Buffers.newDirectDoubleBuffer(newCap);
			
			//copy old verts
			vertsd.limit(oldCap);
			vertsd.position(0);
			nvertsd.put(vertsd);
	
			vlist.position(0);
			nvertsd.put(vlist);
			
			vertsd = nvertsd;
		}
		else
		{
			vlist.position(0);
			vertsd.limit(newCap);
			vertsd.put(vlist);
		}
	}

}
