package org.processmining.visualisation3d.plugin;

import javax.swing.JComponent;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.interactions.JICameraFree;
import org.processmining.visualisation3d.scene.JSOSimpleCube;
import org.processmining.visualisation3d.scene.JSOSimpleGrid;
import org.processmining.visualisation3d.scenemanagement.JoglScene;


/*
 * This is the entry class of the comparative visualisation plug-in.
 * */
@Plugin(
	name = "ThreeDee Test", 
	parameterLabels = { "ThreeDeeViz", "canceller" }, 
	returnLabels = { "Test" }, 
	returnTypes = { JComponent.class }, 
	userAccessible = true, 
	help = "This plug-in compares the differences between different process models"
)
@Visualizer
public class ThreeDeeTestViz {

	@UITopiaVariant(
		affiliation = "Queensland University of Technology", 
		author = "Erik Poppe", 
		email = "erik.poppe@qut.edu.au"
	)
    @PluginVariant(requiredParameterLabels = { 0, 1 })
	public JComponent threeDeeMe(final UIPluginContext context, ThreeDeeViz tdv, ProMCanceller canceller)
	{	
		//System.out.println("threeDeeMe");		

		JoglCamera cam = new JoglCamera();
		JoglScene scene = BuildScene(cam);
		
		JoglPanel panel = new JoglPanel();
		panel.setScene(scene);
		panel.setCamera(cam);
		
		JICameraFree camInteraction = new JICameraFree(cam);
		
		panel.getInteractionManager().addInteraction(camInteraction);
		
		return panel;
	}

	private JoglScene BuildScene(JoglCamera cam) 
	{
		System.out.println("Creating scene...");
		
		//hack! need to load this before the first JoglTextRenderer is loaded
		//JoglResourceManager.AddResource("DefaultFont", new DefaultFontData());
		
		JoglScene scene = new JoglScene();
		//scene.obj = new JSOQuad();
		//scene.obj = new JSOCube();
		scene.addSceneObject(new JSOSimpleGrid(true, true, cam));
		scene.addSceneObject(new JSOSimpleCube());
		
//		FontFormat ff = new FontFormat();
//		ff.width = 5.0f;
//		ff.height = 5.0f;
//		
//		JoglTextRenderer tr = new JoglTextRenderer();
//		tr.setString("Test");
//		scene.addSceneObject(tr);
		
		return scene;
	}
		
	
	
}
