package org.processmining.visualisation3d.scene;

import java.util.Vector;

import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.gui.JoglGuiObjectCollection;
import org.processmining.visualisation3d.materials.MaterialGuiBase;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JSOGraphicalUserInterface extends JoglSceneObject
{
	private static MaterialGuiBase matG = new MaterialGuiBase();
	
	private JoglPanel panel;
	
	private JoglGuiObjectCollection guiObjs = new JoglGuiObjectCollection();
	
	public JSOGraphicalUserInterface(JoglPanel p)
	{
		panel = p;
	}
	
	public void AddGuiObject(JoglGuiObject jgo)
	{
		guiObjs.add(jgo);
		jgo.OnAddToGui(this);
	}
	
	public void RemoveGuiObject(JoglGuiObject jgo)
	{
		jgo.OnRemoveFromGui(this);
		guiObjs.remove(jgo);
	}
	
	public double getAspect()
	{
		return getWidth() / getHeight();
	}
	
	public double getWidth()
	{
		return panel.getWidth();
	}
	
	public double getHeight()
	{
		return panel.getHeight();
	}
	
	public JoglGuiObject GetObjectFromPos(float x, float y)
	{
		JoglVectord2 pos = GetNDCPos(x, y);
		float posx = (float)pos.getX();
		float posy = (float)pos.getY();
		
		//picking order is inverse of drawing order!
		for(int i = guiObjs.size() - 1; i >= 0; i--)
		{
			JoglGuiObject jgo = guiObjs.get(i);
			if (jgo.IsInside(posx, posy))
			{
				return jgo;
			}
		}
		return null;
	}
	
	public JoglGuiObject GetObjectFromPos(double x, double y)
	{
		JoglVectord2 pos = GetNDCPos(x, y);
		float posx = (float)pos.getX();
		float posy = (float)pos.getY();
		
		//picking order is inverse of drawing order!
		for(int i = guiObjs.size() - 1; i >= 0; i--)
		{
			JoglGuiObject jgo = guiObjs.get(i);
			if (jgo.IsInside(posx, posy))
			{
				return jgo;
			}
		}
		return null;
	}
	
	public Vector<JoglGuiObject> GetObjectsFromPos(float x, float y)
	{
		JoglVectord2 pos = GetNDCPos(x, y);
		float posx = (float)pos.getX();
		float posy = (float)pos.getY();
		
		//picking order is inverse of drawing order!
		Vector<JoglGuiObject> lst = new Vector<JoglGuiObject>();
		for(int i = guiObjs.size() - 1; i >= 0; i--)
		{
			JoglGuiObject jgo = guiObjs.get(i);
			if (jgo.IsInside(posx, posy))
			{
				//return jgo;
				lst.add(jgo);
			}
		}
		return lst;
	}
	
	public Vector<JoglGuiObject> GetObjectsFromPos(double x, double y)
	{
		JoglVectord2 pos = GetNDCPos(x, y);
		float posx = (float)pos.getX();
		float posy = (float)pos.getY();
		
		//picking order is inverse of drawing order!
		Vector<JoglGuiObject> lst = new Vector<JoglGuiObject>();
		for(int i = guiObjs.size() - 1; i >= 0; i--)
		{
			JoglGuiObject jgo = guiObjs.get(i);
			if (jgo.IsInside(posx, posy))
			{
				//return jgo;
				lst.add(jgo);
			}
		}
		return lst;
	}
	
	//get normalized device coordinates
	public JoglVectord2 GetNDCPos(float x, float y)
	{
//		float posx = x / (float)panel.getWidth();
//		float posy = y / (float)panel.getHeight();
		float posx = x;
		float posy = y;
		
		posx = (posx * 2.0f) - 1.0f;
		posy = ((1.0f - posy) * 2.0f) - 1.0f;
		
		return new JoglVectord2(posx, posy);
	}
	
	public JoglVectord2 GetNDCPos(double x, double y)
	{
//		double posx = x / (double)panel.getWidth();
//		double posy = y / (double)panel.getHeight();
		double posx = x;
		double posy = y;
		
		posx = (posx * 2.0f) - 1.0f;
		posy = ((1.0f - posy) * 2.0f) - 1.0f;
		
		return new JoglVectord2(posx, posy);
	}
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		//JoglRenderTarget rt = pipe.target;
		
		//revert projection and transformation
		pipe.transform.Push();
		pipe.transform.LoadIdentity();
		
		pipe.projection.Push();
		pipe.projection.LoadIdentity();
		
		pipe.PushPipelineState(matG);
		
		for (JoglGuiObject jgo : guiObjs)
		{
			jgo.DrawObject(pipe);
		}

		pipe.PopPipelineState(matG);

		//reinstate projection and transformation
		pipe.projection.Pop();
		
		pipe.transform.Pop();
	}
}
