package org.processmining.visualisation3d.scenemanagement;

import java.util.Vector;

import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.IMouseProcessor;
import org.processmining.visualisation3d.collision.JoglAABB;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCameraLensPerspective;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;

/*
 * A 3D scene that contains all the objects to render and a camera to determine the view into the scene
 */
public class JoglScene 
{
	protected IMouseProcessor iman = null;
	protected JSOGraphicalUserInterface gui = null;
	protected Vector<JoglSceneObject> objs = new Vector<JoglSceneObject>();
	
	protected boolean bDrawGUI = true;
	
	private JoglVectord3 backgroundColor = new JoglVectord3(0.0,0.0,0.0);
	
	public JoglVectord3 getBackgroundColor()
	{
		return backgroundColor;
	}
	
	public void setBackgroundColor(float r, float g, float b)
	{
		backgroundColor = new JoglVectord3(r, g, b);
	}
	
	public boolean ShouldDrawGui()
	{
		return bDrawGUI;
	}
	
	public void setDrawGui(boolean bVal)
	{
		bDrawGUI = bVal;
	}
	
//	public JSOGraphicalUserInterface getGUI()
//	{
//		return gui;
//	}
	
	protected void UpdateCameraLens(JoglCamera c, double aspect)
	{
		c.setCameraLens(new JoglCameraLensPerspective(45.0, aspect));
	}
	
	public void addSceneObject(JoglSceneObject jso)
	{
		if (jso instanceof JSOGraphicalUserInterface)
		{
			gui = (JSOGraphicalUserInterface)jso;
		}
		objs.add(jso);
	}
	
	public void removeSceneObject(JoglSceneObject jso)
	{
		if (jso == gui)
		{
			gui = null;
		}
		objs.remove(jso);
	}
	
	public <T extends JoglSceneObject> T findSceneObject(Class<T> c)
	{
		for (JoglSceneObject ji : objs)
		{
			if (c.isInstance(ji))
			{
				return (T)ji;
			}
			
			T res = ji.findSceneObject(c);
			if (res != null)
			{
				return res;
			}
		}
		return null;
	}
	
	public <T extends JoglSceneObject> Vector<T> findSceneObjects(Class<T> c)
	{
		Vector<T> lst = new Vector<T>();
		for (JoglSceneObject ji : objs)
		{
			if (c.isInstance(ji))
			{
				lst.add((T)ji);
			}
			
			Vector<T> res = ji.findSceneObjects(c);
			if (!res.isEmpty())
			{
				lst.addAll(res);
			}
		}
		return lst;
	}
	
	public Vector<JoglAABB> getBoundingBoxes()
	{
		return new Vector<JoglAABB>();
	}
	
	public void Draw(GraphicsPipeline pipe)
	{
		pipe.cam.DrawCamera(pipe);
		
		for (int i = 0; i < objs.size(); i++)
		{
			objs.get(i).DrawObject(pipe);
		}
	}
	
	public void DrawGui(GraphicsPipeline pipe)
	{
		
	}
	
	public void Dispose(GraphicsPipeline pipe)
	{
		objs.clear();
		gui = null;
		iman = null;
	}

}
