package org.processmining.visualisation3d.scenemanagement;

import java.util.Comparator;
import java.util.Vector;

import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.graphicsdatastructures.IZSortable;
import org.processmining.visualisation3d.graphicsdatastructures.ZComparer;

/*
 * A generic 3D object to render in the scene
 */
public class JoglSceneObject implements IZSortable
{
	protected double zpos = 0.0;
	protected Vector<JoglSceneObject> children = new Vector<JoglSceneObject>();
	
	public double getZOrder()
	{
		return zpos;
	}
	
	public void setZOrder(double z)
	{
		zpos = z;
	}
	
	public void addChild(JoglSceneObject jso)
	{
		children.add(jso);
	}
	
	public void removeChild(JoglSceneObject jso)
	{
		children.remove(jso);
	}
	
	public <T extends JoglSceneObject> T findSceneObject(Class<T> c)
	{
		for (JoglSceneObject ji : children)
		{
			if (c.isInstance(ji))
			{
				return (T)ji;
			}
			
			T res = ji.findSceneObject(c);
			if (res != null)
			{
				return res;
			}
		}
		return null;
	}
	
	public <T extends JoglSceneObject> Vector<T> findSceneObjects(Class<T> c)
	{
		Vector<T> lst = new Vector<T>();
		for (JoglSceneObject ji : children)
		{
			if (c.isInstance(ji))
			{
				lst.add((T)ji);
			}

			Vector<T> res = ji.findSceneObjects(c);
			if (!res.isEmpty())
			{
				lst.addAll(res);
			}
		}
		return lst;
	}
	
	public void DrawObject(GraphicsPipeline pipe)
	{
		DrawChildrenSorted(pipe);
	}
	
	protected void DrawChildrenSorted(GraphicsPipeline pipe)
	{
		children.sort(new ZComparer());
		for (JoglSceneObject jso : children)
		{
			jso.DrawObject(pipe);
		}
	}
	
	protected void DrawChildrenSorted(GraphicsPipeline pipe, Comparator<? super JoglSceneObject> comp)
	{
		children.sort(comp);
		for (JoglSceneObject jso : children)
		{
			jso.DrawObject(pipe);
		}
	}
}
