package org.processmining.visualisation3d.utils;

import java.io.File;
import java.io.IOException;

import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.gldatastructures.JoglFrameBufferObject;
import org.processmining.visualisation3d.gldatastructures.JoglMultiSampleFrameBufferObject;
import org.processmining.visualisation3d.gldatastructures.JoglTextureBufferObject;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTargetFramebuffer;

import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.GLException;
import com.jogamp.opengl.util.texture.TextureData;
import com.jogamp.opengl.util.texture.TextureIO;

public class JoglScreenshotCaptureHQ implements IScreenshotCapture 
{
	private String fileName = "C:/Lab/Export/screenshot.png";
	private int resx = 1;
	private int resy = 1;
	private int samples = 1;
	
	public JoglScreenshotCaptureHQ(String fileName, int resx, int resy, int samples)
	{
		this.fileName = fileName;
		this.resx = resx;
		this.resy = resy;
		this.samples = samples;
	}

	public void GrabScreenshot(GraphicsPipeline pipe) 
	{
		JoglMultiSampleFrameBufferObject myfboms = new JoglMultiSampleFrameBufferObject(pipe, resx, resy, samples);
		myfboms.Bind(pipe);
		myfboms.AttachDepthBuffer(pipe, myfboms.CreateRboAttachment(pipe, GL.GL_DEPTH_COMPONENT32));
		myfboms.AttachColorBuffer0(pipe, myfboms.CreateRboAttachment(pipe, GL.GL_RGBA8));

		if (!myfboms.isFramebufferComplete())
		{
			System.out.println("Framebuffer (MS) failed to initialize.");
			return;
		}
		
		myfboms.Unbind(pipe);
		
		JoglFrameBufferObject myfbo = new JoglFrameBufferObject(pipe, resx, resy);
		myfbo.Bind(pipe);
		myfbo.AttachColorBuffer0(pipe, myfbo.CreateTboAttachment(pipe, GL.GL_RGB, 0, GL.GL_RGB, GL.GL_UNSIGNED_BYTE));

		if (!myfbo.isFramebufferComplete())
		{
			System.out.println("Framebuffer failed to initialize.");
			return;
		}
		
		myfbo.Unbind(pipe);
		
		JoglTextureBufferObject tbo = (JoglTextureBufferObject)myfbo.getColorBuffer0();
		JoglRenderTargetFramebuffer jrtb = new JoglRenderTargetFramebuffer(myfboms);
		jrtb.setBackgroundColor(pipe.scene.getBackgroundColor());
		
		pipe.scene.setDrawGui(false);
		jrtb.Bind(pipe);
		jrtb.FitViewport(pipe);

		jrtb.Clear(pipe);

		GraphicsPipeline newPipe = new GraphicsPipeline(pipe);
		newPipe.cam = pipe.cam;
		newPipe.scene = pipe.scene;
		newPipe.target = jrtb;
		
		pipe.scene.Draw(newPipe);
		
		//copy from ms framebuffer!
		myfboms.BlitToFramebuffer(pipe, myfbo);
		
		try 
		{
			TextureData td = tbo.getTexture().getTextureData(pipe);
			TextureIO.write(td, new File(fileName));
			
			System.out.println("Screenshot saved: " + fileName);
		} 
		catch (GLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		myfboms.Bind(pipe, GL2.GL_DRAW_FRAMEBUFFER);

		jrtb.Unbind(pipe);
		pipe.scene.setDrawGui(true);
		
		myfbo.Dispose(pipe);
		myfboms.Dispose(pipe);
	}
}
